@@ -1,7 +1,7 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2007 Douglas L. Theobald
+    Copyright (C) 2004-2008 Douglas L. Theobald
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -153,7 +153,7 @@ CalcBfactC(CoordsArray *cdsA)
             }
         }
 
-        cdsA->coords[i]->bfact_c = 3.0 * occsum / trBS;
+        cdsA->coords[i]->bfact_c = occsum / trBS;
         /*printf("bfact_c[%d] = % f\n", i, cdsA->coords[i]->bfact_c);*/
     }
 }
@@ -436,7 +436,7 @@ MVCovMat(CoordsArray *cdsA)
     const double    idf = 1.0 / (double)(cnum * vlen);
     const Coords  **coords = (const Coords **) cdsA->coords;
     const Coords   *coordsi;
-    double        **MVCovMat;
+    double        **MVCovMat = NULL;
 
     if (cdsA->MVCovMat == NULL)
         MVCovMat = cdsA->MVCovMat = MatInit(3, 3);
@@ -601,7 +601,7 @@ CalcAxCovMat2(CoordsArray *cdsA)
     int             i, j, k, m;
     const int       cnum = cdsA->cnum, vlen = cdsA->vlen;
     double          trace;
-    double         *tmpCoordsi;
+    double         *tmpCoordsi = NULL;
     const Coords   *coordsm;
     const double  **WtMat = (const double **) cdsA->WtMat;
     double        **AxCovMat = cdsA->AxCovMat;
@@ -813,7 +813,7 @@ CalcCovMatFullAx(CoordsArray *cdsA)
         for (j = i; j < vlen; ++j)
             CovMat[j][i] = CovMat[i][j] = CovMat[i][j] / (3.0 * cnum);
 
-    MatDestroy(WtMat);
+    MatDestroy(&WtMat);
 }
 
 
