@@ -1,7 +1,7 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2007 Douglas L. Theobald
+    Copyright (C) 2004-2008 Douglas L. Theobald
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -48,7 +48,7 @@ CovInvLAPACK(CoordsArray *cdsA)
     long int                 info = 0;
     char                     upper = 'U';
     long int                 vlen = (long int) cdsA->vlen;
-    double                 **tmpmat;
+    double                 **tmpmat = NULL;
 
     tmpmat = MatInit(cdsA->vlen, cdsA->vlen);
     for (i = 0; i < cdsA->vlen; ++i)
@@ -88,7 +88,7 @@ CovInvLAPACK(CoordsArray *cdsA)
             cdsA->WtMat[j][i] = cdsA->WtMat[i][j] = tmpmat[i][j];
     }
 
-    MatDestroy(tmpmat);
+    MatDestroy(&tmpmat);
     /*MatPrint(cdsA->WtMat, cdsA->vlen);*/
     /*SCREAMS("LAPACK");*/
 }
@@ -104,8 +104,8 @@ CovInvIndefLAPACK(CoordsArray *cdsA)
     long int                 vlen = (long int) cdsA->vlen;
     long int                 lwork;
     long int                *ipiv;
-    double                  *work;
-    double                  *w;
+    double                  *work = NULL;
+    double                  *w = NULL;
 
     w = (double *) malloc(vlen * sizeof(double));
     work = (double *) malloc(sizeof(double));
@@ -172,7 +172,7 @@ CovInvIndefLAPACK(CoordsArray *cdsA)
 void
 CalcLeleCovMat(CoordsArray *cdsA)
 {
-    double        **MMT;
+    double        **MMT = NULL;
     double          idf, cov_sum;
     int             i, j, k;
 
@@ -224,7 +224,7 @@ CalcLeleCovMat(CoordsArray *cdsA)
     }
     */
 
-    MatDestroy(MMT);
+    MatDestroy(&MMT);
 }
 
 
@@ -391,7 +391,7 @@ CoordsInnerProd2(Coords *coords)
     /* (i x k)(k x j) = (i x j) */
     /* (3 x N)(N x 3) = (3 x 3) */
     int             k;
-    double        **innerprod2;
+    double        **innerprod2 = NULL;
     const double   *x = (const double *) coords->x,
                    *y = (const double *) coords->y,
                    *z = (const double *) coords->z;
@@ -429,7 +429,7 @@ void
 MatInnerProdSymIp(double **mat, const int N)
 {
     int             i, j, k;
-    double        **workmat;
+    double        **workmat = NULL;
     
     workmat = MatInit(N, N);
 
@@ -449,7 +449,7 @@ MatInnerProdSymIp(double **mat, const int N)
         for (j = 0; j < N; ++j)
              mat[i][j] = workmat[i][j];
 
-    MatDestroy(workmat);
+    MatDestroy(&workmat);
 }
 
 
@@ -565,8 +565,8 @@ CalcEDMADistMat(CoordsArray *cdsA)
 
     /* VariancesEDMA((const double *) cdsA->var, cdsA->Dij_matrix, distmat, len); */
 
-    MatDestroy(H);
-    MatDestroy(distmat);
+    MatDestroy(&H);
+    MatDestroy(&distmat);
 }
 
 
@@ -671,8 +671,8 @@ CalcEDMADistMatOcc(CoordsArray *cdsA)
 
     /* VariancesEDMA((const double *) cdsA->var, cdsA->Dij_matrix, distmat, len); */
 
-    MatDestroy(H);
-    MatDestroy(distmat);
+    MatDestroy(&H);
+    MatDestroy(&distmat);
 }
 
 
@@ -688,7 +688,7 @@ void
 LeleCenterMat(double **mat, const int len)
 {
     int             i, j, k;
-    double        **tmpmat;
+    double        **tmpmat = NULL;
     double          off_diagonal, tmp;
 
     /* set up H, the centering/normalizing matrix */
@@ -730,7 +730,7 @@ LeleCenterMat(double **mat, const int len)
         for (j = 0; j < len; ++j)
             mat[i][j] *= -0.5;
 
-    MatDestroy(tmpmat);
+    MatDestroy(&tmpmat);
 }
 
 
@@ -926,7 +926,7 @@ EmbedAveCoords(CoordsArray *cdsA)
     long int                 il, iu, m;
     double                   vl, vu, abstol;
     double                   w0, w1, w2;
-    double                 **z;
+    double                 **z = NULL;
     long int                *isuppz, *iwork;
     long int                 lwork, liwork, negone;
     double                  *work, *w;
@@ -1057,6 +1057,6 @@ EmbedAveCoords(CoordsArray *cdsA)
     free(work);
     free(iwork);
     free(w);
-    MatDestroy(z);
+    MatDestroy(&z);
     free(isuppz);
 }
