@@ -1,7 +1,7 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2007 Douglas L. Theobald
+    Copyright (C) 2004-2008 Douglas L. Theobald
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -193,7 +193,7 @@ CalcLedoitCovMatPar(CoordsArray *cdsA)
 {
     double          m, a2, b2, d2;
     double          factor1, factor2, term1;
-    double        **tmp_covmat;
+    double        **tmp_covmat = NULL;
     int             i, j;
     const int       vlen = cdsA->vlen, cnum = cdsA->cnum;
     double        **CovMat = cdsA->CovMat;
@@ -741,7 +741,7 @@ CalcLedoitVarNewPar(CoordsArray *cdsA)
     m = average((const double *) vars, plen);
 
     /* calculate d^2, equal to the variance across the variances */
-    d2 = variance((const double *) vars, plen, m);
+    d2 = Variance((const double *) vars, plen, m);
 
     if (d2 == 0)
     {
@@ -751,7 +751,7 @@ CalcLedoitVarNewPar(CoordsArray *cdsA)
 
     /* calculate b^2, the average error of the covariance matrix estimate
        this assumes a normal dist, Var(var) =  2 * var^2 * (N-1) / N^2 */
-    b2 = varvariance((const double *) vars, plen, nlen);
+    b2 = VarVariance((const double *) vars, plen, nlen);
 
     /* b^2 = min(b^2, d^2) */
     if (d2 < b2)
@@ -1066,6 +1066,6 @@ CalcSteinVar(CoordsArray *cdsA)
     for (i = 0; i < plen; ++i)
         variances[i] = CovMat[i][i];
 
-    MatDestroy(evecs);
+    MatDestroy(&evecs);
     free(evals);
 }
