@@ -1,7 +1,7 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2007 Douglas L. Theobald
+    Copyright (C) 2004-2008 Douglas L. Theobald
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -29,6 +29,7 @@
 #include <math.h>
 #include <ctype.h>
 #include "Error.h"
+#include "lodmats.h"
 #include "pdbMalloc.h"
 #include "Coords.h"
 #include "PDBCoords.h"
@@ -44,7 +45,7 @@ range_selxn(int index, int *lower, int *upper, int range_num);
 static int
 atom_selxn(char *name, int mode);
 
-static const char atoms0[] = ":CA :P  :";
+static const char atoms0[] = ":CA :C1*:";
 static const char atoms1[] = ":N  :C  :O  :CA :";
 
 static int
@@ -115,7 +116,7 @@ VerifyAlignmentVsPDBs(PDBCoordsArray *pdbA, CoordsArray *baseA, MSA *msa,
     int             i, j, len, aalen;
     const int       cnum = pdbA->cnum;
     const int       seqlen = msa->seqlen;
-    char           *seq;
+    char           *seq = NULL;
 
     for (i = 0; i < cnum; ++i)
     {
@@ -154,7 +155,7 @@ Align2segID(PDBCoordsArray *pdbA)
     const int       cnum = pdbA->cnum;
     int            *map = pdbA->seq2pdb->map;
     MSA            *msa = pdbA->seq2pdb->msa;
-    char           *seq;
+    char           *seq = NULL;
     int             resSeq;
     char            alnindex[5];
 
@@ -284,24 +285,38 @@ GetSeq2PDBMap_old(PDBCoordsArray *pdbA, Seq2PDB *seq2pdb, MSA *msa)
    Then the fxn matches the sequence name in the mapfile with a sequence name in the alignment.
    Given these two matchings and the mapfile, the map[] vector can be constructed.
    Finally, given Coords i, then map[i] = k, where k is the index of the corresponding
-   sequence in the MSA */
+   sequence in the MSA.
+   NB: filename extensions are ignored. */
 static void
 GetSeq2PDBMap(PDBCoordsArray *pdbA, Seq2PDB *seq2pdb, MSA *msa)
 {
     int             i, j, k;
     const int       cnum = pdbA->cnum;
+    char            filename_root[FILENAME_MAX], msaname_root[FILENAME_MAX], seqname_root[FILENAME_MAX];
+
+    /* First set all flags in the MSA to 1, and later reset to 0 if the sequences have a 
+       corresponding PDB file.
+       This allows for sequences in the MSA that aren't being superpositioned. */
+    for (i = 0; i < msa->seqnum; ++i)
+        msa->flag[i] = 1;
 
     for (i = 0; i < cnum; ++i)
     {
         for (j = 0; j < cnum; ++j)
         {
-            if (strncmp(pdbA->coords[i]->filename, seq2pdb->pdbfile_name[j], strlen(seq2pdb->pdbfile_name[j])) == 0)
+            strncpy(filename_root, getroot(pdbA->coords[i]->filename), strlen(pdbA->coords[i]->filename)+1);
+            strncpy(seqname_root, getroot(seq2pdb->pdbfile_name[j]), strlen(seq2pdb->pdbfile_name[j])+1);
+
+            if (strncmp(filename_root, seqname_root, strlen(seqname_root)) == 0)
             {
                 for (k = 0; k < msa->seqnum; ++k)
                 {
-                    if (strncmp(seq2pdb->seqname[j], msa->name[k], strlen(msa->name[k])) == 0) /* now k (alignment) and i (pdb coords) match */
+                    strncpy(msaname_root, getroot(msa->name[k]), strlen(msa->name[k])+1);
+
+                    if (strncmp(seqname_root, msaname_root, strlen(msaname_root)) == 0) /* now k (alignment) and i (pdb coords) match */
                     {
                         pdbA->seq2pdb->map[i] = k;
+                        msa->flag[k] = 0; /* reset msa flag to 0 since this structure exists */
                         break;
                     }
                 }
@@ -338,15 +353,25 @@ DefaultSeq2PDBMap(PDBCoordsArray *pdbA, Seq2PDB *seq2pdb, MSA *msa)
 {
     int             i, j;
     const int       cnum = pdbA->cnum;
+    char            filename_root[FILENAME_MAX], msaname_root[FILENAME_MAX];
+
+    /* First set all flags in the MSA to 1, and later reset to 0 if the sequences have a 
+       corresponding PDB file.
+       This allows for sequences in the MSA that aren't being superpositioned. */
+    for (i = 0; i < msa->seqnum; ++i)
+        msa->flag[i] = 1;
 
     for (i = 0; i < cnum; ++i)
     {
         for (j = 0; j < msa->seqnum; ++j)
         {
-            /* if (strncmp(msa->name[i], pdbA->coords[j]->filename, strlen(pdbA->coords[j]->filename)) == 0) */
-            if (strncmp(pdbA->coords[i]->filename, msa->name[j], strlen(pdbA->coords[i]->filename)) == 0)
+            strncpy(filename_root, getroot(pdbA->coords[i]->filename), strlen(pdbA->coords[i]->filename)+1);
+            strncpy(msaname_root, getroot(msa->name[j]), strlen(msa->name[j])+1);
+
+            if (strncmp(filename_root, msaname_root, FILENAME_MAX) == 0)
             {
                 seq2pdb->map[i] = j; /* now j (alignment) and i (pdb coords) match */
+                msa->flag[j] = 0; /* reset msa flag to 0 since this structure exists */
                 break;
             }
         }
@@ -374,11 +399,15 @@ GetSingletons(int *singletons, MSA *msa)
         count = 0;
         for (j = 0; j < msa->seqnum; ++j)
         {
-            if (msa->seq[j][i] != '-')
+            /* The flag for each sequence was set/unset in GetSeq2PDBMap() or DefaultSeq2PDBMap()
+               If flag is set to 1, we ignore it (because it has no PDB structural counterpart) */
+            if (msa->seq[j][i] != '-' && msa->flag[j] == 0)
                 ++count;
         }
 
-        if (count == 1)
+        /* This flags all columns that have either only one or NO residues,
+           and both need to be excluded from the superposition */
+        if (count == 1 || count == 0)
             singletons[i] = 1;
         else
             singletons[i] = 0;
@@ -386,6 +415,32 @@ GetSingletons(int *singletons, MSA *msa)
 }
 
 
+/* check for ubiqs, i.e. columns with no gaps */
+static void
+GetUbiqs(int *ubiqs, MSA *msa)
+{
+    int             i, j, count;
+
+    for (i = 0; i < msa->seqlen; ++i)
+    {
+        count = 0;
+        for (j = 0; j < msa->seqnum; ++j)
+        {
+            /* The flag for each sequence was set/unset in GetSeq2PDBMap() or DefaultSeq2PDBMap()
+               If flag is set to 1, we ignore it (because it has no PDB structural counterpart) */
+            if (msa->seq[j][i] == '-' && msa->flag[j] == 0)
+                ++count;
+        }
+//printf("\ncount[%d] = %d\n", i, count);
+        /* if ANY gaps were counted, set flag. 
+           NOTE that this does not do anything for the other
+           flags -- they are the same as when ubiqs was passed */
+        if (count > 0)
+            ubiqs[i] = 1;
+    }
+}
+
+
 static void
 GetCoordsFrAlignment(PDBCoordsArray *pdbA, CoordsArray *baseA,
                      Seq2PDB *seq2pdb, MSA *msa,
@@ -438,7 +493,6 @@ GetCoordsFrAlignment(PDBCoordsArray *pdbA, CoordsArray *baseA,
 
 						++m;
 						++n;
-						++p;
 					}
 					else /* is a gap */
 					{
@@ -452,21 +506,18 @@ GetCoordsFrAlignment(PDBCoordsArray *pdbA, CoordsArray *baseA,
 						baseA->coords[j]->b[n]       = 99.99;
 
 						++n;
-						++p;
 					}
 				}
 				else /* out of range */
 				{
-					if (msa->seq[k][p] != '-') /* is not a gap */
+					if (msa->seq[k][p] != '-') /* not a gap */
 					{
 						++m;
-						++p;
-					}
-					else /* is a gap */
-					{
-						++p;
 					}
+
 				}
+
+				++p;
 /* 				            if (j == cnum-1) */
 /*                                 printf("\n2 m:%d(%d) msa->seq[%d(%d)][%d(%d)] = %c", */
 /*                                        m, pdbA->coords[j]->vlen, k, cnum, n, alignlen, msa->seq[k][n]); */
@@ -673,7 +724,8 @@ Align2MSA(PDBCoordsArray *pdbA, CoordsArray *baseA, char *msafile_name, char *ma
     int             i, alnlen, vlen;
     const int       cnum = pdbA->cnum;
     MSA            *msa;
-    int            *singletons;
+    int            *singletons = NULL;
+    Algorithm      *algo = baseA->algo;
 
     if (mapfile_name == NULL)
     {
@@ -701,10 +753,19 @@ Align2MSA(PDBCoordsArray *pdbA, CoordsArray *baseA, char *msafile_name, char *ma
     }
 
     alnlen = msa->seqlen;
-
     singletons = malloc(alnlen * sizeof(int));
+
+    if (mapfile_name != NULL)
+        GetSeq2PDBMap(pdbA, pdbA->seq2pdb, msa);
+    else
+        DefaultSeq2PDBMap(pdbA, pdbA->seq2pdb, msa);
+
     GetSingletons(singletons, msa);
-    vlen = ParseSelxns(pdbA, baseA->algo->selection, alnlen, singletons, baseA->algo->revsel);
+    if (algo->missing == 1)
+        GetUbiqs(singletons, msa);
+//    for (i=0;i<alnlen;++i)
+//        printf("singleton[%3d]:%d\n", i, singletons[i]);
+    vlen = ParseSelxns(pdbA, algo->selection, alnlen, singletons, algo->revsel);
 
 /*     printf("\n%d %d %d", msa->seqlen, alnlen, vlen); */
 
@@ -724,14 +785,14 @@ Align2MSA(PDBCoordsArray *pdbA, CoordsArray *baseA, char *msafile_name, char *ma
 //    printf("\n%d %d %d\n", msa->seqlen, alnlen, vlen);
 
     /* allocate a CoordsArray based on this alignment */
-    if (baseA->algo->atoms == 0)
+    if (algo->atoms == 0)
+    {
         CoordsArrayAlloc(baseA, cnum, vlen);
-/*     else if (baseA->algo->atoms == 1) */
-/*         CoordsArrayAlloc(baseA, cnum, 4 * alnlen); */
+    }
     else
     {
         fprintf(stderr,
-                "\n  ERROR683: atom selection must be CAs or backbone for superpositioning with a sequence alignment\n");
+                "\n  ERROR683: atom selection must be CAs for superpositioning with a sequence alignment\n");
         PrintTheseusTag();
         exit(EXIT_FAILURE);
     }
@@ -739,12 +800,7 @@ Align2MSA(PDBCoordsArray *pdbA, CoordsArray *baseA, char *msafile_name, char *ma
     for (i = 0; i < cnum; ++i)
         strncpy(baseA->coords[i]->filename, pdbA->coords[i]->filename, FILENAME_MAX - 1);
 
-    if (mapfile_name != NULL)
-        GetSeq2PDBMap(pdbA, pdbA->seq2pdb, msa);
-    else
-        DefaultSeq2PDBMap(pdbA, pdbA->seq2pdb, msa);
-
-    VerifyAlignmentVsPDBs(pdbA, baseA, msa, baseA->algo->atoms, pdbA->seq2pdb->map);
+    VerifyAlignmentVsPDBs(pdbA, baseA, msa, algo->atoms, pdbA->seq2pdb->map);
     GetCoordsFrAlignment(pdbA, baseA, pdbA->seq2pdb, msa, vlen, alnlen, singletons);
 
     free(singletons);
@@ -791,7 +847,7 @@ CalcRotationsOcc(CoordsArray *cdsA)
             /* rmsd from mean would usually need a 2 in denom,\
                but this is already deviation from mean,
                since structure #2 is the average structure */
-            printf(" %5d %8.3f %13.3f \n",
+            printf("CalcRotationsOcc:%5d %8.3f %13.3f \n",
                    i+1,
                    coords[i]->wRMSD_from_mean,
                    deviation);
@@ -802,8 +858,6 @@ CalcRotationsOcc(CoordsArray *cdsA)
 }
 
 
-#include "lodmats.h"
-
 /* char aa1[] = "ARNDCQEGHILKMFPSTWYV"; */
 /* char aa3[] = "ALAARGASNASPCYSGLNGLUGLYHISILELEULYSMETPHEPROSERTHRTRPTYRVAL"; */
 
@@ -811,7 +865,7 @@ void
 pdb2fst(PDBCoordsArray *pdbA)
 {
     int             i, j, count, aaindex;
-    char           *pindex;
+    char           *pindex = NULL;
     FILE           *fp;
     char            outfile[FILENAME_MAX];
 
@@ -829,8 +883,9 @@ pdb2fst(PDBCoordsArray *pdbA)
         {
             //printf("ATOM %4d:%4d \'%s\'\n", i, j, pdbA->coords[i]->name[j]);
 
-            if (strncmp(pdbA->coords[i]->record[j], "ATOM", 4) == 0 &&
-                (strncmp(pdbA->coords[i]->name[j], "CA ", 3) == 0 || strncmp(pdbA->coords[i]->name[j], "P  ", 3) == 0) &&
+            if ( (strncmp(pdbA->coords[i]->record[j], "ATOM  ", 6) == 0 || strncmp(pdbA->coords[i]->record[j], "HETATM", 6) == 0)
+                &&
+                (strncmp(pdbA->coords[i]->name[j], "CA ", 3) == 0 || strncmp(pdbA->coords[i]->name[j], "C1*", 3) == 0) &&
                 (pdbA->coords[i]->altLoc[j] == ' ' || pdbA->coords[i]->altLoc[j] == 'A'))
             {
 				if (count % 72 == 0)
