@@ -1,7 +1,7 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2007 Douglas L. Theobald
+    Copyright (C) 2004-2008 Douglas L. Theobald
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -927,7 +927,7 @@ Mixture(CoordsArray *cdsA, PDBCoordsArray *pdbA)
 
     for (i = 0; i < mixn; ++i)
     {
-        CoordsArrayDestroy(mixA[i]);
+        CoordsArrayDestroy(&mixA[i]);
         free(probs[i]);
     }
 
@@ -936,9 +936,9 @@ Mixture(CoordsArray *cdsA, PDBCoordsArray *pdbA)
     free(aveprobs);
     free(slope);
     free(oldprobs);
-    PDBCoordsArrayDestroy(pdb2A);
-    MatDestroy(newprobs);
-    MatDestroy(vars);
+    PDBCoordsArrayDestroy(&pdb2A);
+    MatDestroy(&newprobs);
+    MatDestroy(&vars);
 
     return(count);
 }
@@ -1115,7 +1115,7 @@ Mixture_pth(CoordsArray *cdsA, PDBCoordsArray *pdbA)
 
     for (i = 0; i < mixn; ++i)
     {
-        CoordsArrayDestroy(mixA[i]);
+        CoordsArrayDestroy(&mixA[i]);
         free(probs[i]);
     }
 
@@ -1124,12 +1124,12 @@ Mixture_pth(CoordsArray *cdsA, PDBCoordsArray *pdbA)
     free(aveprobs);
     free(slope);
     free(oldprobs);
-    MatDestroy(newprobs);
+    MatDestroy(&newprobs);
     for (i = 0; i < mixn; ++i)
         free(mixdata[i]);
     free(mixdata);
     free(callThd);
-    MatDestroy(vars);
+    MatDestroy(&vars);
 
     return(count);
 }
@@ -1595,9 +1595,9 @@ MultiPoseMix(CoordsArray *baseA, const double *probs, double *vars)
     memcpy(vars, scratchA->var, vlen * sizeof(double));
     memcpy(vars, baseA->var, vlen * sizeof(double));
 
-    CoordsArrayDestroy(scratchA);
+    CoordsArrayDestroy(&scratchA);
     free(evals);
-    MatDestroy(lastmat);
+    MatDestroy(&lastmat);
 
     return(round);
 }
