@@ -1,7 +1,7 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2007 Douglas L. Theobald
+    Copyright (C) 2004-2008 Douglas L. Theobald
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -64,7 +64,7 @@ CalcPCA(CoordsArray *cdsA)
     double                 **CovMat = cdsA->CovMat;
     double                   sum, runsum;
     PDBCoords               *pdbave;
-    char                     pcafile_name[256], numstring[5];
+    char                     pcafile_name[256];
     FILE                    *pcavecs_fp, *pcastats_fp;
     double                   biggest, bstick;
     char                     aster;
@@ -179,10 +179,12 @@ CalcPCA(CoordsArray *cdsA)
         for (j = 0; j < vlen; ++j)
             cdsA->pdbA->avecoords->tempFactor[j] = cdsA->pcamat[pcanum - 1 - i][j] * (99.99 / biggest);
 
-        strcpy(pcafile_name, mystrcat(cdsA->algo->rootname, "_pc"));
+        sprintf(pcafile_name, "%s_pc%d_ave.pdb", cdsA->algo->rootname, i+1);
+
+        //strcpy(pcafile_name, mystrcat(cdsA->algo->rootname, "_pc"));
         //pcafile_name[11] = '\0';
-        strncat(pcafile_name, itoa(i+1, &numstring[0], 10), 5);
-        strncat(pcafile_name, "_ave.pdb", 8);
+        //strncat(pcafile_name, itoa(i+1, &numstring[0], 10), 5);
+        //strncat(pcafile_name, "_ave.pdb", 8);
 
         WriteAvePDBCoordsFile(cdsA->pdbA, pcafile_name);
     }
@@ -209,7 +211,7 @@ Calc3NPCA(CoordsArray *cdsA)
     double                 **evecs = NULL, *evals = NULL;
     double                   sum, runsum;
     PDBCoords               *pdbave = NULL;
-    char                     pcafile_name[256], numstring[5];
+    char                     pcafile_name[256];
     FILE                    *pcavecs_fp = NULL, *pcastats_fp = NULL;
     double                   biggest, bstick;
     char                     aster;
@@ -319,9 +321,11 @@ Calc3NPCA(CoordsArray *cdsA)
         for (j = 0; j < vlen; ++j)
             cdsA->pdbA->avecoords->tempFactor[j] = evecs[pcanum - 1 - i][j] * (99.99 / biggest);
 
-        strncpy(pcafile_name, mystrcat(cdsA->algo->rootname, "_pc"), 11);
-        pcafile_name[11] = '\0';
-        strncat(pcafile_name, itoa(i+1, &numstring[0], 10), 5);
+        sprintf(pcafile_name, "%s_pc%d", cdsA->algo->rootname, i+1);
+
+        //strncpy(pcafile_name, mystrcat(cdsA->algo->rootname, "_pc"), 11);
+        //pcafile_name[11] = '\0';
+        //strncat(pcafile_name, itoa(i+1, &numstring[0], 10), 5);
 
         WriteAvePDBCoordsFile(cdsA->pdbA, pcafile_name);
     }
@@ -332,7 +336,7 @@ Calc3NPCA(CoordsArray *cdsA)
     fclose(pcastats_fp);
     fclose(pcavecs_fp);
 
-    MatDestroy(mat);
+    MatDestroy(&mat);
 }
 
 
@@ -455,7 +459,7 @@ WritePCAFile(PDBCoordsArray *parray, CoordsArray *cdsA, const char *outfile_root
     const int       cvlen = cdsA->vlen;
     const int       pvlen = parray->vlen;
     double          tempFactor;
-    char            numstring[5];
+    //char            numstring[5];
     char            covcor_str[16] = "correlation";
 
     /* find largest absolute value in the eigenvector PCA */
@@ -523,11 +527,13 @@ WritePCAFile(PDBCoordsArray *parray, CoordsArray *cdsA, const char *outfile_root
             }
         }
 
-        strncpy(pcafile_name, outfile_root, strlen(outfile_root));
-        pcafile_name[strlen(outfile_root)] = '\0';
-        strncat(pcafile_name, "_pc", 4);
-        strncat(pcafile_name, itoa(i+1, &numstring[0], 10), 5);
-        strcat(pcafile_name, ".pdb");
+        sprintf(pcafile_name, "%s_pc%d.pdb", outfile_root, i+1);
+
+/*         strncpy(pcafile_name, outfile_root, strlen(outfile_root)); */
+/*         pcafile_name[strlen(outfile_root)] = '\0'; */
+/*         strncat(pcafile_name, "_pc", 4); */
+/*         strncat(pcafile_name, itoa(i+1, &numstring[0], 10), 5); */
+/*         strcat(pcafile_name, ".pdb"); */
 
         pdbfile = fopen(pcafile_name, "w");
         if (pdbfile ==NULL)
@@ -576,7 +582,6 @@ WriteModelPCAFile(PDBCoordsArray *parray, CoordsArray *cdsA, const char *outfile
     const int       pvlen = parray->vlen;
     const int       cnum = cdsA->cnum;
     double          tempFactor;
-    char            numstring[5];
     char            covcor_str[16] = "correlation";
     const int       pcanum = cdsA->cnum;
 
@@ -603,11 +608,13 @@ WriteModelPCAFile(PDBCoordsArray *parray, CoordsArray *cdsA, const char *outfile
                 parray->coords[j]->tempFactor[k] = tempFactor;
         }
 
-        strncpy(pcafile_name, outfile_root, strlen(outfile_root));
-        pcafile_name[strlen(outfile_root)] = '\0';
-        strncat(pcafile_name, "_mod_pca", 8);
-        strncat(pcafile_name, itoa(i+1, &numstring[0], 10), 5);
-        strcat(pcafile_name, ".pdb");
+        sprintf(pcafile_name, "%s_mod_pca%d.pdb", outfile_root, i+1);
+
+/*         strncpy(pcafile_name, outfile_root, strlen(outfile_root)); */
+/*         pcafile_name[strlen(outfile_root)] = '\0'; */
+/*         strncat(pcafile_name, "_mod_pca", 8); */
+/*         strncat(pcafile_name, itoa(i+1, &numstring[0], 10), 5); */
+/*         strcat(pcafile_name, ".pdb"); */
 
         pdbfile = fopen(pcafile_name, "w");
         if (pdbfile ==NULL)
@@ -659,7 +666,6 @@ WritePCAMorphFile(PDBCoordsArray *parray, CoordsArray *cdsA, const char *outfile
     const int       cvlen = 3 * cdsA->vlen;
     const int       pvlen = parray->vlen;
     PDBCoords      *pcacoords;
-    char            numstring[5];
     char            covcor_str[16] = "correlation";
 
     pcacoords = PDBCoordsInit();
@@ -726,12 +732,14 @@ CalcFullCovMat(cdsA);
                 break;
         }
 
-        strncpy(pcafile_name, outfile_root, strlen(outfile_root));
-        pcafile_name[strlen(outfile_root)] = '\0';
-        strncat(pcafile_name, "_pca", 4);
-        strncat(pcafile_name, itoa(i+1, &numstring[0], 10), 5);
-        strncat(pcafile_name, "_morph", 6);
-        strcat(pcafile_name, ".pdb");
+        sprintf(pcafile_name, "%s_pca%d_morph.pdb", outfile_root, i+1);
+
+/*         strncpy(pcafile_name, outfile_root, strlen(outfile_root)); */
+/*         pcafile_name[strlen(outfile_root)] = '\0'; */
+/*         strncat(pcafile_name, "_pca", 4); */
+/*         strncat(pcafile_name, itoa(i+1, &numstring[0], 10), 5); */
+/*         strncat(pcafile_name, "_morph", 6); */
+/*         strcat(pcafile_name, ".pdb"); */
 
         pdbfile = fopen(pcafile_name, "w");
         if (pdbfile == NULL)
@@ -758,7 +766,7 @@ CalcFullCovMat(cdsA);
         fclose(pdbfile);
     }
 
-    PDBCoordsDestroy(pcacoords);
+    PDBCoordsDestroy(&pcacoords);
     PCADestroy(cdsA);
 }
 
@@ -775,7 +783,7 @@ WritePCAProjections(PDBCoordsArray *parray, CoordsArray *cdsA, const char *outfi
     const int       cvlen = 3 * cdsA->vlen;
     const int       pvlen = parray->vlen;
     PDBCoords      *pcacoords;
-    char            numstring[5], tmpstring[5];
+    //char            numstring[5], tmpstring[5];
 
     pcacoords = PDBCoordsInit();
     PDBCoordsAlloc(pcacoords, pvlen);
@@ -847,13 +855,15 @@ CalcFullCovMat(cdsA);
                     break;
             }
 
-            strncpy(pcafile_name, outfile_root, strlen(outfile_root));
-            pcafile_name[strlen(outfile_root)] = '\0';
-            strncat(pcafile_name, "_pca", 4);
-            strncat(pcafile_name, itoa(i+1, &numstring[0], 10), 5);
-            sprintf(tmpstring, "_%+3.1f", c);
-            strncat(pcafile_name, tmpstring, 5);
-            strcat(pcafile_name, ".pdb");
+            sprintf(pcafile_name, "%s_pca%d_%+3.1f.pdb", outfile_root, i+1, c);
+
+/*             strncpy(pcafile_name, outfile_root, strlen(outfile_root)); */
+/*             pcafile_name[strlen(outfile_root)] = '\0'; */
+/*             strncat(pcafile_name, "_pca", 4); */
+/*             strncat(pcafile_name, itoa(i+1, &numstring[0], 10), 5); */
+/*             sprintf(tmpstring, "_%+3.1f", c); */
+/*             strncat(pcafile_name, tmpstring, 5); */
+/*             strcat(pcafile_name, ".pdb"); */
 
             pdbfile = fopen(pcafile_name, "w");
             if (pdbfile == NULL)
@@ -878,6 +888,6 @@ CalcFullCovMat(cdsA);
         }
     }
 
-    PDBCoordsDestroy(pcacoords);
+    PDBCoordsDestroy(&pcacoords);
     PCADestroy(cdsA);
 }
