@@ -1,7 +1,7 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2007 Douglas L. Theobald
+    Copyright (C) 2004-2008 Douglas L. Theobald
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -336,7 +336,7 @@ Calc3NPCA(CoordsArray *cdsA)
     fclose(pcastats_fp);
     fclose(pcavecs_fp);
 
-    MatDestroy(mat);
+    MatDestroy(&mat);
 }
 
 
