@@ -1,7 +1,7 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2007 Douglas L. Theobald
+    Copyright (C) 2004-2008 Douglas L. Theobald
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -631,9 +631,9 @@ CalcLAPACKSVD(double **Rmat, double **Umat, double *sigma, double **VTmat)
 /*     printf("\n imag %f %f %f", wi[0], wi[1], wi[2]); */
 /*     free(wr); */
 /*     free(wi); */
-/*     MatDestroy(vl); */
-/*     MatDestroy(vr); */
-/*     MatDestroy(tmpmat); */
+/*     MatDestroy(&vl); */
+/*     MatDestroy(&vr); */
+/*     MatDestroy(&tmpmat); */
 
 /*     free(work); */
 }
@@ -691,7 +691,7 @@ ProcLAPACKSVDvan(const Coords *coords1, const Coords *coords2, double **rotmat,
     CalcRvan(coords1, coords2, Rmat);
     CalcLAPACKSVD(Rmat, Umat, sigma, VTmat);
     det = CalcRotMat(rotmat, Umat, VTmat);
-    term2 = CalcInnProdNormRot(coords1, (const double **) rotmat);
+    term2 = CalcInnProdNorm(coords1);
     sumdev = term1 + term2;
 
 /*     VerifyRotMat(rotmat, 1e-5); */
