@@ -1,7 +1,7 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2007 Douglas L. Theobald
+    Copyright (C) 2004-2008 Douglas L. Theobald
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -42,11 +42,7 @@
 #include "DLTmath.h"
 #include "internmat.h"
 #include "RandCoords.h"
-
-extern char tc_RED[];
-extern char tc_GREEN[];
-extern char tc_NC[];
-extern char tc_CYAN[];
+#include "termcol.h"
 
 
 /*======================================================================*
@@ -216,7 +212,7 @@ RandRotCoordsArray(CoordsArray *cdsA)
         RotateCoordsIp(cdsA->coords[i], (const double **) rotmat);
     }
 
-    MatDestroy(rotmat);
+    MatDestroy(&rotmat);
 }
 
 
@@ -277,6 +273,93 @@ quadratic_solver(const double a, const double b, const double c,
 
 void
 RandCoords(CoordsArray *cdsA)
+{
+    int            i, j, k, m, protlen = cdsA->vlen;
+    double       **covmat = MatInit(protlen, protlen);
+    double        *p = calloc(protlen, sizeof(double));
+    double       **randv = MatInit(protlen, 3), **randn = MatInit(protlen, 3);
+    double        *diag = malloc(protlen * sizeof(double));
+    double        *invdiag = malloc(protlen * sizeof(double));
+    Coords       **coords = cdsA->coords;
+    Coords        *mcoords = cdsA->coords[0];
+    double         scale = 1.0;
+
+//     for (i = 0; i < protlen; ++i)
+//         for (j = 0; j < protlen; ++j)
+//             if (i != j)
+//                 covmat[i][j] = uniform_dev(-1.0, 1.0);
+
+     for (i = 0; i < protlen; ++i)
+         covmat[i][i] = 1.0;
+
+/*     for (i = 0; i < protlen; ++i) */
+/*         printf("%-3d %f\n", i+1, covmat[i][i]); */
+
+     for (i = 0; i < protlen; ++i)
+         diag[i] = invgamma_dev(cdsA->algo->param[0], cdsA->algo->param[1]);
+
+    PrintCovMatGnuPlot((const double **) covmat, protlen, mystrcat(cdsA->algo->rootname, "_cor.mat"));
+
+    for (i = 0; i < protlen; ++i)
+        printf("%-3d %f\n", i+1, diag[i]);
+
+    for (i = 0; i < protlen; ++i)
+        invdiag[i] = 1.0 / diag[i];
+
+    CenMass(mcoords);
+    ApplyCenterIp(mcoords);
+
+    if (mcoords->innerprod == NULL)
+        mcoords->innerprod = MatInit(protlen, protlen);
+
+    CoordsInnerProd(mcoords);
+    PrintCovMatGnuPlot((const double **) mcoords->innerprod, protlen, mystrcat(cdsA->algo->rootname, "_mean.mat"));
+
+    /* LAPACK dpotrf calculates the Cholesky decomposition, here upper triangular */
+    dpotrf_opt_dest(covmat, protlen);
+
+    for (i = cdsA->cnum - 1; i >=0; --i)
+    {
+        RandFillMat(randn, protlen, 3, cdsA->algo->random, 0, 0, 0);
+
+        memset(&randv[0][0], 0, protlen * 3 * sizeof(double));
+        for (m = 0; m < 3; ++m)
+            for (j = 0; j < protlen; ++j)
+                for (k = 0; k < protlen; ++k) /* because covmat is upper diagonal, lower is all zeros */        
+                    randv[j][m] += covmat[k][j] * randn[k][m];
+
+/*         for (j = 0; j < protlen; ++j) */
+/*             fprintf(fp, "\n %-4d % 11f % 11f % 11f", j+1, randv[j][0], randv[j][1], randv[j][2]); */
+
+        for (j = 0; j < protlen; ++j)
+        {
+            scale = RandScale(diag[j], cdsA->algo->random, 0, 0);
+            /* printf("\n%d %f", j, scale); */
+
+            coords[i]->x[j] = mcoords->x[j] +
+                              randv[j][0] * scale;
+            coords[i]->y[j] = mcoords->y[j] +
+                              randv[j][1] * scale;
+            coords[i]->z[j] = mcoords->z[j] +
+                              randv[j][2] * scale;
+/*             if (j == 0) */
+/*                 printf("\n%-3d % 8.3f % 8.3f % 8.3f", */
+/*                        i, coords[i]->x[j], coords[i]->y[j], coords[i]->z[j]); */
+        }
+    }
+
+/*     fclose(fp); */
+
+    MatDestroy(&covmat);
+    MatDestroy(&randn);
+    MatDestroy(&randv);
+    free(p);
+    free(diag);
+}
+
+
+void
+RandCoords_2sdf(CoordsArray *cdsA)
 {
     int            i, j, k, m, protlen = cdsA->vlen;
     double       **covmat = MatInit(protlen, protlen);
@@ -328,7 +411,7 @@ RandCoords(CoordsArray *cdsA)
 /*     CalcCovMat(cdsA); */  
 /*     CalcLedoitCovMat(cdsA); */
 
-    /* memcpy(&covmat[0][0], &internmat[0][0], protlen * protlen * sizeof(double)); */
+    memcpy(&covmat[0][0], &internmat[0][0], protlen * protlen * sizeof(double));
 
 /*     for (i = 0; i < protlen; ++i) */
 /*         for (j = 0; j < protlen; ++j) */
@@ -360,13 +443,13 @@ RandCoords(CoordsArray *cdsA)
 /*     for (i = 0; i < protlen; ++i) */
 /*         covmat[i][i] = 3.0; */
 
-    for (i = 0; i < protlen; ++i)
-        for (j = 0; j < protlen; ++j)
-            if (i != j)
-                covmat[i][j] = uniform_dev(-1.0, 1.0);
+//     for (i = 0; i < protlen; ++i)
+//         for (j = 0; j < protlen; ++j)
+//             if (i != j)
+//                 covmat[i][j] = uniform_dev(-1.0, 1.0);
 
-    for (i = 0; i < protlen; ++i)
-        covmat[i][i] = 1.0;
+//     for (i = 0; i < protlen; ++i)
+//         covmat[i][i] = 1.0;
 
 /*     for (i = 0; i < protlen; ++i) */
 /*         printf("%-3d %f\n", i+1, covmat[i][i]); */
@@ -376,13 +459,13 @@ RandCoords(CoordsArray *cdsA)
 
 /*     memcpy(&diag[0], &simevals2[0], protlen *sizeof(double)); */
 
-    for (i = 0; i < protlen; ++i)
-    {
-        /* diag[i] = covmat[i][i] = invgamma_dev(3.468e-03, 3.779e-01); */
-        diag[i] = invgamma_dev(cdsA->algo->param[0], cdsA->algo->param[1]);
-    }
+//     for (i = 0; i < protlen; ++i)
+//     {
+//         /* diag[i] = covmat[i][i] = invgamma_dev(3.468e-03, 3.779e-01); */
+//         diag[i] = invgamma_dev(cdsA->algo->param[0], cdsA->algo->param[1]);
+//     }
 
-    CorMat2CovMat(covmat, diag, protlen);
+//     CorMat2CovMat(covmat, diag, protlen);
 
 /*     for (i = 0; i < protlen; ++i) */
 /*         for (j = 0; j < protlen; ++j) */
@@ -506,9 +589,9 @@ RandCoords(CoordsArray *cdsA)
 
 /*     fclose(fp); */
 
-    MatDestroy(covmat);
-    MatDestroy(randn);
-    MatDestroy(randv);
+    MatDestroy(&covmat);
+    MatDestroy(&randn);
+    MatDestroy(&randv);
     free(p);
     free(diag);
 }
@@ -673,7 +756,7 @@ PDBCoordsArray
     {
         strncpy(dirpdbfile_name, pdbdir, FILENAME_MAX - 1);
         strncat(dirpdbfile_name, pdbfile_name, FILENAME_MAX - strlen(pdbfile_name) - 1);
-        
+
         pdbfile = fopen(dirpdbfile_name, "r");
 
         if (pdbfile == NULL)
@@ -722,7 +805,7 @@ PDBCoordsArray
     }
 
     ReadPDBCoords(pdbfile_name, pdbA, 0, 1);
-    
+
     for (i = 1; i < cnum; ++i)
         PDBCoordsCopyAll(pdbA->coords[i], (const PDBCoords *) pdbA->coords[0]);
 
