@@ -6,9 +6,9 @@ then
 elif [ "${1}" = "gryphon" ]
 then
     remotehost="gryphon.biochem.brandeis.edu";
-elif [ "${1}" = "yellow" ]
+elif [ "${1}" = "hippocerf" ]
 then
-    remotehost="yellow.biochem.brandeis.edu";
+    remotehost="hippocerf.biochem.brandeis.edu";
 elif [ "${1}" = "wren" ]
 then
     remotehost="xray.bmb.colostate.edu";
@@ -28,7 +28,7 @@ else
     remotehost="www.molevo.org";
 fi
 
-FILES="*.[ch] Makefile* make.inc archive* AUTHORS COPYING INSTALL.txt LICENSE README theseus.1 theseus_align examples/ lib/ libDLTutils/ libdistfit/ libdltmath/ libdssplite/ libmsa/";
+FILES="*.[ch] Makefile* make.inc archive* AUTHORS COPYING INSTALL.txt LICENSE README make_dist.sh theseus.1 theseus_man.pdf theseus_align examples/ lib/ libDLTutils/ libdistfit/ libdltmath/ libdssplite/ libmsa/";
 
 DATE=$(date +%s);
 TARFILE="theseus${DATE}.tgz";
