@@ -1,7 +1,7 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2007 Douglas L. Theobald
+    Copyright (C) 2004-2008 Douglas L. Theobald
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -40,19 +40,19 @@
 
 #include "distfit_local.h"
 
-#define VERSION "0.83"
-#define DISTNUM 24
+#define VERSION "0.85"
+#define DISTNUM 21
 
 /* array of function names */
 char
 dist_name[DISTNUM][32] = 
 {
 	"Normal",      "Laplace",   "EVD",        "Logistic",
-	"Cauchy",      "Uniform",   "VonMises",   "Weibull",
-	"Gamma",       "NegBinom",  "ChisqrGen",  "Lognormal",
+	"Cauchy",      "Uniform",   "Weibull",
+	"Gamma",       "ChisqrGen",  "Lognormal",
 	"Invgamma",    "Invgauss",  "Recinvgauss", "Pareto",
 	"Exponential", "Invchisqr", "Chisqr",      "Chi",
-	"Maxwell",     "Beta",      "BetaSym",     "Binomial"
+	"Maxwell",     "Beta",      "BetaSym",     //"Binomial"
 };
 
 /* array of function pointers */
@@ -60,11 +60,11 @@ DistFit
 distfit_func[DISTNUM] = 
 {
 	&normal_fit,    &laplace_fit,   &EVD_fit,         &logistic_fit,
-	&cauchy_fit,    &uniform_fit,   &vonmises_fit,    &weibull_fit,
-	&gamma_fit,     &negbinom_fit,  &chisqrgen_fit,   &lognormal_fit,
+	&cauchy_fit,    &uniform_fit,   &weibull_fit,
+	&gamma_fit,     &chisqrgen_fit,   &lognormal_fit,
 	&invgamma_fit,  &invgauss_fit,  &recinvgauss_fit, &pareto_fit,
 	&exp_fit,       &invchisqr_fit, &chisqr_fit,      &chi_fit,
-	&maxwell_fit,   &beta_fit,      &betasym_fit,     &binomial_fit
+	&maxwell_fit,   &beta_fit,      &betasym_fit,     //&binomial_fit
 };
 
 /* array of function pointers */
@@ -72,66 +72,66 @@ DistDev
 distdev_func[DISTNUM] = 
 {
 	&normal_dev,    &laplace_dev,   &EVD_dev,         &logistic_dev,
-	&cauchy_dev,    &uniform_dev,   &vonmises_dev,    &weibull_dev,
-	&gamma_dev,     &negbinom_dev,  &chisqrgen_dev,   &lognormal_dev,
+	&cauchy_dev,    &uniform_dev,   &weibull_dev,
+	&gamma_dev,     &chisqrgen_dev, &lognormal_dev,
 	&invgamma_dev,  &invgauss_dev,  &recinvgauss_dev, &pareto_dev,
 	&exp_dev,       &invchisqr_dev, &chisqr_dev,      &chi_dev,
-	&maxwell_dev,   &beta_dev,      &betasym_dev,     &binomial_dev
+	&maxwell_dev,   &beta_dev,      &betasym_dev,     //&binomial_dev
 };
 
 DistPDF
 distpdf_func[DISTNUM] = 
 {
 	&normal_pdf,    &laplace_pdf,   &EVD_pdf,         &logistic_pdf,
-	&cauchy_pdf,    &uniform_pdf,   &vonmises_pdf,    &weibull_pdf,
-	&gamma_pdf,     &negbinom_pdf,  &chisqrgen_pdf,   &lognormal_pdf,
+	&cauchy_pdf,    &uniform_pdf,   &weibull_pdf,
+	&gamma_pdf,     &chisqrgen_pdf,   &lognormal_pdf,
 	&invgamma_pdf,  &invgauss_pdf,  &recinvgauss_pdf, &pareto_pdf,
 	&exp_pdf,       &invchisqr_pdf, &chisqr_pdf,      &chi_pdf,
-	&maxwell_pdf,   &beta_pdf,      &betasym_pdf,     &binomial_pdf
+	&maxwell_pdf,   &beta_pdf,      &betasym_pdf,     //&binomial_pdf
 };
 
 DistCDF
 distcdf_func[DISTNUM] = 
 {
 	&normal_cdf,    &laplace_cdf,   &EVD_cdf,         &logistic_cdf,
-	&cauchy_cdf,    &uniform_cdf,   &vonmises_cdf,    &weibull_cdf,
-	&gamma_cdf,     &negbinom_cdf,  &chisqrgen_cdf,   &lognormal_cdf,
+	&cauchy_cdf,    &uniform_cdf,   &weibull_cdf,
+	&gamma_cdf,     &chisqrgen_cdf,   &lognormal_cdf,
 	&invgamma_cdf,  &invgauss_cdf,  &recinvgauss_cdf, &pareto_cdf,
 	&exp_cdf,       &invchisqr_cdf, &chisqr_cdf,      &chi_cdf,
-	&maxwell_cdf,   &beta_cdf,      &betasym_cdf,     &binomial_cdf
+	&maxwell_cdf,   &beta_cdf,      &betasym_cdf,     //&binomial_cdf
 };
 
 DistSDF
 distsdf_func[DISTNUM] = 
 {
 	&normal_sdf,    &laplace_sdf,   &EVD_sdf,         &logistic_sdf,
-	&cauchy_sdf,    &uniform_sdf,   &vonmises_sdf,    &weibull_sdf,
-	&gamma_sdf,     &negbinom_sdf,  &chisqrgen_sdf,   &lognormal_sdf,
+	&cauchy_sdf,    &uniform_sdf,   &weibull_sdf,
+	&gamma_sdf,     &chisqrgen_sdf,   &lognormal_sdf,
 	&invgamma_sdf,  &invgauss_sdf,  &recinvgauss_sdf, &pareto_sdf,
 	&exp_sdf,       &invchisqr_sdf, &chisqr_sdf,      &chi_sdf,
-	&maxwell_sdf,   &beta_sdf,      &betasym_sdf,     &binomial_sdf
+	&maxwell_sdf,   &beta_sdf,      &betasym_sdf,     //&binomial_sdf
 };
 
 DistInt
 distint_func[DISTNUM] = 
 {
 	&normal_int,    &laplace_int,   &EVD_int,         &logistic_int,
-	&cauchy_int,    &uniform_int,   &vonmises_int,    &weibull_int,
-	&gamma_int,     &negbinom_int,  &chisqrgen_int,   &lognormal_int,
+	&cauchy_int,    &uniform_int,   &weibull_int,
+	&gamma_int,     &chisqrgen_int,   &lognormal_int,
 	&invgamma_int,  &invgauss_int,  &recinvgauss_int, &pareto_int,
 	&exp_int,       &invchisqr_int, &chisqr_int,      &chi_int,
-	&maxwell_int,   &beta_int,      &betasym_int,     &binomial_int
+	&maxwell_int,   &beta_int,      &betasym_int,     //&binomial_int
 };
 
 int
 dist_pnum[DISTNUM] = 
 {
 	2, 2, 2, 2,
-	2, 2, 2, 2,
-	2, 2, 2, 2,
+	2, 2, 2,
+	2, 2, 2,
 	2, 2, 2, 2,
 	1, 1, 1, 1,
-	1, 2, 1, 2
+	1, 2, 1, //2
 };
 
 
@@ -139,11 +139,11 @@ int
 dist_nonneg[DISTNUM] = 
 {
 	0, 0, 0, 0,
-	0, 0, 0, 1,
-	1, 1, 1, 0,
+	0, 0, 1,
+	1, 1, 0,
 	0, 0, 0, 0,
 	1, 0, 1, 1,
-	1, 0, 0, 1
+	1, 0, 0, //1
 };
 
 
@@ -151,22 +151,22 @@ int
 dist_posdef[DISTNUM] = 
 {
 	0, 0, 0, 0,
-	0, 0, 0, 0,
-	0, 0, 0, 1,
+	0, 0, 0,
+	0, 0, 1,
 	1, 1, 1, 1,
 	0, 1, 0, 0,
-	0, 0, 0, 0
+	0, 0, 0, //0
 };
 
 int
 dist_beta[DISTNUM] = 
 {
 	0, 0, 0, 0,
+	0, 0, 0,
+	0, 0, 0,
 	0, 0, 0, 0,
 	0, 0, 0, 0,
-	0, 0, 0, 0,
-	0, 0, 0, 0,
-	0, 1, 1, 0
+	0, 1, 1, //0
 };
 
 DFParams        DfParams;
@@ -235,38 +235,34 @@ ParseCmdLine(int *argc, char **argv[])
                         dfparams->distindex = 4;
                     else if (strncmp(optarg, "uniform", 4) == 0)
                         dfparams->distindex = 5;
-                    else if (strncmp(optarg, "vonmises", 3) == 0)
-                        dfparams->distindex = 6;
                     else if (strncmp(optarg, "weibull", 4) == 0)
-                        dfparams->distindex = 7;
+                        dfparams->distindex = 6;
                     else if (strncmp(optarg, "gamma", 4) == 0)
-                        dfparams->distindex = 8;
-                    else if (strncmp(optarg, "negbinom", 4) == 0)
-                        dfparams->distindex = 9;
+                        dfparams->distindex = 7;
                     else if (strncmp(optarg, "chisqrgen", 7) == 0)
-                        dfparams->distindex = 10;
+                        dfparams->distindex = 8;
                     else if (strncmp(optarg, "lognormal", 4) == 0)
-                        dfparams->distindex = 11;
+                        dfparams->distindex = 9;
                     else if (strncmp(optarg, "invgamma", 5) == 0)
-                        dfparams->distindex = 12;
+                        dfparams->distindex = 10;
                     else if (strncmp(optarg, "invgauss", 5) == 0)
-                        dfparams->distindex = 13;
+                        dfparams->distindex = 11;
                     else if (strncmp(optarg, "recinvgauss", 3) == 0)
-                        dfparams->distindex = 14;
+                        dfparams->distindex = 12;
                     else if (strncmp(optarg, "pareto", 4) == 0)
-                        dfparams->distindex = 15;
+                        dfparams->distindex = 13;
                     else if (strncmp(optarg, "exponential", 4) == 0)
-                        dfparams->distindex = 16;
+                        dfparams->distindex = 14;
                     else if (strncmp(optarg, "invchisqr", 4) == 0)
-                        dfparams->distindex = 17;
+                        dfparams->distindex = 15;
                     else if (strncmp(optarg, "chisqr", 4) == 0)
-                        dfparams->distindex = 18;
+                        dfparams->distindex = 16;
                     else if (strncmp(optarg, "chi", 3) == 0)
-                        dfparams->distindex = 19;
+                        dfparams->distindex = 17;
                     else if (strncmp(optarg, "maxwell", 3) == 0)
-                        dfparams->distindex = 20;
+                        dfparams->distindex = 18;
                     else if (strncmp(optarg, "beta", 4) == 0)
-                        dfparams->distindex = 21;
+                        dfparams->distindex = 19;
                     else
                     {
                         printf("\n  Bad -d \"%s\" \n", optarg);
@@ -1247,7 +1243,7 @@ double
 {
     int            i, numscanned;
     FILE          *listfile = NULL;
-    double        *array;
+    double        *array = NULL;
     double         columns[16];
     char           line[512], *comment;
     int            ch;
@@ -1281,7 +1277,10 @@ double
     while(fgets(line, 512, listfile) != NULL && i < *length)
     {
         if (line[0] == '#')
+        {
+            --(*length);
             continue;
+        }
 
         comment = strchr(line, '#');
 
