@@ -1,7 +1,7 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2007 Douglas L. Theobald
+    Copyright (C) 2004-2008 Douglas L. Theobald
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -72,13 +72,13 @@ void
 MatPrintRec(double **matrix, const int n, const int m);
 
 void
-MatDestroy(double **matrix);
+MatDestroy(double ***matrix_ptr);
 
 double
 **MatInit(const int rows, const int cols);
 
 void
-MatIntDestroy(int **matrix);
+MatIntDestroy(int ***matrix_ptr);
 
 int
 **MatIntInit(const int rows, const int cols);
@@ -87,7 +87,7 @@ Matrix3D
 *Mat3DInit(const int rows, const int cols, const int depth);
 
 void
-Mat3DDestroy(Matrix3D *matrix3d);
+Mat3DDestroy(Matrix3D **matrix3d_ptr);
 
 double
 MatFrobNorm(const double **mat1, const double **mat2, const int row, const int col);
@@ -579,12 +579,30 @@ invgamma_logL(const double *data, const int num, const double b, const double c)
 double
 invgamma_fit(const double *data, const int num, double *b, double *c, double *logL);
 
+double
+ExpXn(const double b, const double c, const double n);
+
+double
+ExpLogXn(const double b, const double c, const double n);
+
+double
+ExpInvXn(const double b, const double c, const double xn1);
+
+double
+invgamma_EMsmall_fit(const double *data, const int num, const int missing, double *b, double *c, double *logL);
+
 double
 invgamma_bayes_fit(const double *data, const int num, double *b, double *c, double *logL);
 
 double
 invgamma1_fit(const double *data, const int num, double *b, double *nullp, double *logL);
 
+double
+invgamma_fixed_c_EM_fit(const double *data, const int num, const int missing, double *b, const double c, double *logL);
+
+double
+invgamma_fixed_c_EM_fit_bayes(const double *data, const int num, const int missing, double *b, const double c, double *logL);
+
 double
 invgamma_fit_guess(const double *data, const int num, double *b, double *c, double *logL);
 
@@ -1276,10 +1294,10 @@ double
 average(const double *data, const int dim);
 
 double
-variance(const double *data, const int dim, const double mean);
+Variance(const double *data, const int dim, const double mean);
 
 double
-varvariance(const double *data, const int ddim, const int sdim);
+VarVariance(const double *data, const int ddim, const int sdim);
 
 int
 moments(const double *data, const int dim,
