@@ -1,7 +1,7 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2007 Douglas L. Theobald
+    Copyright (C) 2004-2008 Douglas L. Theobald
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -28,6 +28,8 @@
 #include <stdlib.h>
 #include <string.h>
 #include <float.h>
+#include <gsl/gsl_sf_gamma.h>
+#include <gsl/gsl_sf_psi.h>
 #include "DLTmath.h"
 #include "statistics.h"
 #include "gamma_dist.h"
@@ -225,7 +227,7 @@ gamma_sdf(const double x, const double b, const double c)
            /* 1.0 - InGamma(c, x / b)-q */
            /* gammq(1e8, 1e8)) );*/
 
-    return(InGammaQ(c, x / b));
+    return(gsl_sf_gamma_inc_Q(c, x / b));
 }
 
 
@@ -239,7 +241,7 @@ gamma_int(const double x, const double y, const double b, const double c)
 double
 gamma_logL(const double b, const double c)
 {
-    return(-(lgamma(c) + log(b) + (1.0 - c) * s_PolyGamma(c, 1) + c));
+    return(-(lgamma(c) + log(b) + (1.0 - c) * gsl_sf_psi(c) + c));
 }
 
 
@@ -358,7 +360,7 @@ gamma_Stacyfit2(const double *z, const int num, double *b, double *c)
 /*     double Y = exp(X); */
 /*     while (L > 1e-8) */
 /*     { */
-/*         Y += L * mysign(X - s_PolyGamma(Y, 1)); */
+/*         Y += L * mysign(X - gsl_sf_psi(Y)); */
 /*         L *= 0.5; */
 /*     } */
 /*  */
@@ -378,18 +380,18 @@ gamma_Stacyfit2(const double *z, const int num, double *b, double *c)
 static void
 evalgammaML(const double logterm, const double c, double *fx, double *dfx)
 {
-    *fx = log(c) - s_PolyGamma(c, 1) + logterm;
-/*     printf("\n c:%e log(c):% e -s_PolyGamma(c,2):% e sum:% e logterm:%e", */
-/*            c, log(c), - s_PolyGamma(c, 1), log(c) - s_PolyGamma(c, 1), logterm); */
+    *fx = log(c) - gsl_sf_psi(c) + logterm;
+/*     printf("\n c:%e log(c):% e -gsl_sf_psi_1(c):% e sum:% e logterm:%e", */
+/*            c, log(c), - gsl_sf_psi(c), log(c) - gsl_sf_psi(c), logterm); */
 /*     fflush(NULL); */
-    *dfx = 1.0/c - s_PolyGamma(c, 2);
+    *dfx = 1.0/c - gsl_sf_psi_1(c);
 }
 
-
+/* NB******* this is wrong, there is a square root omitted */
 /* Find the Bayesian point estimate, the posterior mode, assuming a
    noninformative improper Jeffreys' prior for the shape and scale parameters:
 
-       (c * Digamma(c) - 1) / b
+       (c * Trigamma(c) - 1) / b
 
    Jeffreys prior for the gamma from:
 
@@ -419,10 +421,12 @@ evalgammaBayes(const double logterm, const double c, double *fx, double *dfx)
 {
     double         pg1, pg2, pg3, pg4, cpg2m1;
 
-    pg1 = s_PolyGamma(c, 1);
-    pg2 = s_PolyGamma(c, 2);
-    pg3 = s_PolyGamma(c, 3);
-    pg4 = s_PolyGamma(c, 4);
+    pg1 = gsl_sf_psi(c);
+    pg2 = gsl_sf_psi_1(c);
+    pg3 = gsl_sf_psi_n(2, c);
+    pg4 = gsl_sf_psi_n(3, c);
+//    pg3 = s_PolyGamma(c, 3);
+//    pg4 = s_PolyGamma(c, 4);
     cpg2m1 = c * pg2 - 1.0;
 
     *fx = log(c + 1.0) - pg1 + logterm - 0.5 * (pg2 + c * pg3) / cpg2m1;
@@ -430,11 +434,60 @@ evalgammaBayes(const double logterm, const double c, double *fx, double *dfx)
 }
 
 
+/* Find the Bayesian point estimate, the posterior mode, assuming a
+   noninformative improper Bernardo reference prior for the shape and scale parameters:
+
+       sqrt(Trigamma(c) - 1/c) / b
+
+   Reference prior for the gamma from (p298):
+
+   Liseo, Brunero (1993)
+   "Elimination of Nuisance Parameters with Reference Priors."
+   Biometrika 80(2): 295-304.
+
+   The joint log probability, with the prior, is:
+   
+	   L(b,c|x) = (c-1) \Sum(ln(x_i)) - (1/b)\Sum(x) - Nc ln(b) - N \LnGamma(c)
+				  -(1/2) ln(\Trigamma(c) - 1/c)
+   
+   We must find root of:
+
+                                                           \Quadgamma(c) + 1/c^2
+	   F1 = \Sum(ln(x_i)) - N ln(b) - N Digamma(c) - (1/2) ---------------------  = 0
+                                                            \Trigamma(c) - 1/c
+
+   where the first derivative (dF1/dc) is:
+
+                   /                                                                        \
+                   | (\Quadgamma(c) + 1/c^2)^2    2 - c^3 Pentagamma(c)                      |
+	   F1' = (1/2) | ------------------------- + -----------------------   - 2N \Trigamma(c) |
+	               | (\Trigamma(c) - 1/c)^2       c^2 (c \Trigamma(c) - 1)                   |
+                   \                                                                        /
+
+   Derivatives verified with Mathematica.
+*/
+static void
+evalgammaBayesRef(const double sumlnx, const double b, const double c, const int num, double *fx, double *dfx)
+{
+    double         DiG, TriG, QuaG, PenG;
+
+    DiG  = gsl_sf_psi(c);
+    TriG = gsl_sf_psi_1(c);
+    QuaG = gsl_sf_psi_n(2, c);
+    PenG = gsl_sf_psi_n(3, c);
+//    QuaG = s_PolyGamma(c, 3);
+//    PenG = s_PolyGamma(c, 4);
+
+    *fx = sumlnx - num * log(b) - num * DiG - 0.5 * (QuaG + 1.0/(c*c)) / (TriG - 1.0/c);
+    *dfx = 0.5 * ( (QuaG + 1.0/(c*c))*(QuaG + 1.0/(c*c)) / (TriG - 1.0/c)*(TriG - 1.0/c) + (2.0 - c*c*c*PenG) / (c*c*(TriG - 1.0)) - 2.0*num*TriG);
+}
+
+
 /* static  */void
 evalgammaEM(const double logterm, const double b, const double c, double *fx, double *dfx)
 {
-    *fx = - s_PolyGamma(c, 1) + logterm - log(b) ;
-    *dfx = - s_PolyGamma(c, 2);
+    *fx = - gsl_sf_psi(c) + logterm - log(b) ;
+    *dfx = - gsl_sf_psi_1(c);
 }
 
 
@@ -837,6 +890,134 @@ gamma_bayes_fit_no_stats(const double *data, const int num, double *b, double *c
 }
 
 
+/* Bayesian fit with a Bernardo reference prior on the scale and shape params (see evalgammaBayesRef()) */
+void
+gamma_bayes_ref_fit_no_stats(const double *data, const int num, double *b, double *c)
+{
+    double          sumx, /* var,  */logterm, logdata, fx, dfx, fxdfx, guess_b, guess_c;
+    int             i, maxiter = 500;
+    double          tol = FLT_EPSILON;
+
+    fx = 0.0;
+    dfx = DBL_MAX;
+
+    sumx = 0.0;
+    for (i = 0; i < num; ++i)
+    {
+        if (data[i] <= 0.0)
+        {
+            fprintf(stderr, "\n ERROR345: Gamma distributed data must be > 0.0 ");
+            fprintf(stderr, "\n ERROR345: Aborting calculation \n\n");
+            exit(EXIT_FAILURE);
+        }
+        else if (!isfinite(data[i]))
+        {
+            fprintf(stderr, "\n ERROR346: Gamma distributed data must be finite(!), data[%d] = %f ",
+                    i, data[i]);
+            fprintf(stderr, "\n ERROR346: Aborting calculation \n\n");
+            exit(EXIT_FAILURE);
+        }
+        else
+        {
+            sumx += data[i];
+        }
+    }
+
+/*     var = 0.0; */
+/*     for (i = 0; i < num; ++i) */
+/*     { */
+/*         if (data[i] > tol) */
+/*             var += mysquare(data[i] - ave); */
+/*     } */
+/*     var /= newnum; */
+/*  */
+/*     guess_b = *b = var / ave; */
+/*     guess_c = *c = mysquare(ave) / var; */
+
+    gamma_Stacyfit2(data, num, b, c);
+
+    if (*b < DBL_EPSILON)
+        *b = DBL_EPSILON;
+
+    if (*c < DBL_EPSILON)
+        *c = DBL_EPSILON;
+
+    if (*b > FLT_MAX)
+        *b = FLT_MAX;
+
+    if (*c > FLT_MAX)
+        *c = FLT_MAX;
+
+    guess_b = *b;
+    guess_c = *c;
+    /* Bayesian maximum posterior probability fit.
+       Use Newton-Raphson to find ML estimate of c.
+       
+       The log likelihood, with the Jeffreys' prior, is:
+       
+           L(b,c|x) = c \Sum(ln(x_i)) - (1/b)\Sum(x) - Nc ln(b) - N \Gamma(c)
+                      -N ln(b) - (N/2) ln(c \Trigamma(c) - 1)           
+    */
+    logdata = 0.0;
+    for (i = 0; i < num; ++i)
+        logdata += log(data[i]);
+
+    logterm = logdata - log(sumx);
+    /* printf("% 10.6e % 10.6e\n", fabs(logterm), FLT_EPSILON * 0.5 * (logdata + log(ave))); */
+    if (fabs(logterm) < fabs(FLT_EPSILON * 0.5 * (logdata + log(sumx))))
+    {
+        fprintf(stderr, "\n ERROR349: data are too similar to fit to a gamma distribution \n\n");
+        *c = FLT_MAX;
+        *b = sumx / (*c * num);
+        //return;
+    }
+
+/*     printf("\n            b             c            fx           dfx        fx/dfx"); */
+/*     printf("\n% 10.6e % 10.6e % 10.6e % 10.6e % 10.6e", */
+/*         *b, *c, fx, dfx, fx/dfx); */
+
+    for (i = 0; i < maxiter; ++i)
+    {
+        //evalgammaBayes(logterm, *c, &fx, &dfx);
+        evalgammaBayesRef(logdata, *b, *c, num, &fx, &dfx);
+
+        fxdfx = fx / dfx;
+        *c -= fxdfx; /* Newton-Raphson correction */
+        /* *b = ave / *c; */
+
+        if (*c <= DBL_EPSILON)
+            *c = 0.5 * (fxdfx + *c);
+
+         if (fabs(fxdfx) < tol * *c)
+            break; /* success */
+
+/*         printf("\n%3d % 10.6e % 10.6e % 10.6e % 10.6e % 10.6e", */
+/*                i, ave / *c, *c, fx, dfx, fx/dfx); */
+    }
+/*         printf("\ndone: %3d % 10.6e % 10.6e % 10.6e % 10.6e % 10.6e", */
+/*                i, ave / *c, *c, fx, dfx, fx/dfx); */
+
+    /* plug in result to get b */
+    *b = sumx / (*c * num + 1);
+    /* printf("\n newton():% e   invpsi(): % e", *c, invpsi(logterm - log(*b))); */
+    if (i == maxiter)
+    {
+		printf("\n WARNING02: Newton-Raphson failed to converge in gamma_fit()\n");
+
+        for (i = 0; i < num; ++i)
+            printf("%3d %f\n", i, data[i]);
+
+		printf("            b             c            fx           dfx        fx/dfx       logterm\n");
+		printf("% 10.6e % 10.6e % 10.6e % 10.6e % 10.6e % 10.6e\n",
+			   *b, *c, fx, dfx, fx/dfx, logterm);
+
+		fflush(NULL);
+/*         *b = guess_b; */
+/*         *c = guess_c; */
+    }
+}
+
+
 double
 gamma1_fit(const double *data, const int num, double *b, double *nullp, double *logL)
 {
@@ -1054,8 +1235,8 @@ static void
 evalgammaML_minc_opt(const double logterm, const double minc, const double s, double *fx, double *dfx)
 {
     const double    term = minc + s*s;
-    *fx = log(term) - s_PolyGamma(term, 1) - logterm;
-    *dfx = (2.0 * s / term) - (2.0 * s * s_PolyGamma(term, 2));
+    *fx = log(term) - gsl_sf_psi(term) - logterm;
+    *dfx = (2.0 * s / term) - (2.0 * s * gsl_sf_psi_1(term));
 }
 
 
