@@ -1,7 +1,7 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2007 Douglas L. Theobald
+    Copyright (C) 2004-2008 Douglas L. Theobald
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -1308,6 +1308,8 @@ InvSymEigenOp(double **invmat, const double **mat, long int n,
 }
 
 
+/* Entire decomposition, evals and evecs */
+/* eigensym evals are small to large */
 void
 eigensym(const double **mat, double *evals, double **evecs, long int n)
 {
@@ -1341,6 +1343,9 @@ eigensym(const double **mat, double *evals, double **evecs, long int n)
 }
 
 
+/* Entire decomposition, evals and evecs */
+/* eigensym evals are small to large */
+/* exactly the same as eigensym above, but work space is passed */
 void
 eigensym2(const double **mat, double *evals, double **evecs, long int n, double *work)
 {
@@ -1448,7 +1453,7 @@ eigengen(const double **mat, double *evals, double **evecs, long int n)
 
     free(wi);
     free(work);
-    MatDestroy(tmpmat);
+    MatDestroy(&tmpmat);
 }
 
 
