@@ -1,7 +1,7 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2007 Douglas L. Theobald
+    Copyright (C) 2004-2008 Douglas L. Theobald
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -237,7 +237,7 @@ dgesvd_opt_save(double **a, long int m, long int n,
            &info);
 
     free(work);
-    MatDestroy(mat);
+    MatDestroy(&mat);
 
     return((int) info);
 }
@@ -778,7 +778,7 @@ dsyevr_opt_save(const double **amat, long int n,
     free(work);
     free(iwork);
     free(isuppz);
-    MatDestroy(mat);
+    MatDestroy(&mat);
 
     return((int) info);
 }
@@ -968,8 +968,8 @@ pseudoinv_sym(double **inmat, double **outmat, long int n, const double tol)
 
     free(work);
     free(s);
-    MatDestroy(u);
-    MatDestroy(vt);
+    MatDestroy(&u);
+    MatDestroy(&vt);
 
     return(info);
 }
