@@ -1,7 +1,7 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2007 Douglas L. Theobald
+    Copyright (C) 2004-2008 Douglas L. Theobald
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -180,17 +180,88 @@ XPLORcorrections(PDBCoords *pdbcoords, int record)
 }
 
 
+static void
+ScanPDBLine(char *buff, PDBCoords *cp, int j)
+{
+    char           *endline = NULL;
+
+    endline = strpbrk(&buff[54], "\r\n\f");
+    if (endline != NULL) /* kill the newline, if there */
+        *endline = '\0';
+
+    if (!isdigit(buff[12]) && !isspace(buff[12])) /* for nonstandard PDB files, with atom name in wrong column (shifted one to the left, such as with AMBER8) */
+    {
+        /*         1         2         3         4         5         6         7         8
+        12345678901234567890123456789012345678901234567890123456789012345678901234567890
+        ATOM    145  N   VAL A  25      32.433  16.336  57.540  1.00 11.92      A1   N
+        ATOM    146  CA  VAL A  25      31.132  16.439  58.160  1.00 11.85      A1   C  
+        ATOM  15088 1HG1 ILE    28      -3.430   4.303  -6.057  1.00  0.00           H  
+        ATOM   7580 HD23 LEU  1724     111.285  90.889 -61.535                      
+        */
+                                                      /* #  COLs  LEN  DATA TYPE    FIELD      DEFINITION */
+        sscanf(&buff[0],  "%6c",   cp->record[j]);    /* 1  0-5   6    Record name  "ATOM  " */
+        sscanf(&buff[6],  "%5u",  &cp->serial[j]);    /* 2  6-10  5    Integer      serial     Atom serial number. */
+        sscanf(&buff[12], "%3c",   cp->name[j]);      /* 4  12-14 4(3) Atom         name       Atom name. -- unofficial, nonstandard PDB (AMBER8 format, hosers) */
+        sscanf(&buff[15], "%1c",  &cp->Hnum[j]);      /* 3  15    1      hydrogen number, (!official). */
+        sscanf(&buff[16], "%1c",  &cp->altLoc[j]);    /* 5  16    1    Character    altLoc     Alternate location indicator. */
+        sscanf(&buff[17], "%3c",   cp->resName[j]);   /* 6  17-19 3    Residue name resName    Residue name. */
+        sscanf(&buff[20], "%1c",  &cp->xchainID[j]);  /* 7  20    1    Character    xchainID   Chain identifier (!official). */
+        sscanf(&buff[21], "%1c",  &cp->chainID[j]);   /* 8  21    1    Character    chainID    Chain identifier. */
+        sscanf(&buff[22], "%4d",  &cp->resSeq[j]);    /* 9  22-25 4    Integer      resSeq     Residue sequence number. */
+        sscanf(&buff[26], "%1c",  &cp->iCode[j]);     /* 10 26    1    AChar        iCode      Code for insertion of residues. */
+        sscanf(&buff[30], "%8lf", &cp->x[j]);         /* 11 30-37 8    Real(8.3)    x          Orthogonal coordinates for X */
+        sscanf(&buff[38], "%8lf", &cp->y[j]);         /* 12 30-37 8    Real(8.3)    y          Orthogonal coordinates for Y */
+        sscanf(&buff[46], "%8lf", &cp->z[j]);         /* 13 30-37 8    Real(8.3)    z          Orthogonal coordinates for Z */
+        sscanf(&buff[54], "%6lf", &cp->occupancy[j]); /* 14 54-59 6    Real(6.2)    occupancy  Occupancy. */
+        sscanf(&buff[60], "%6lf", &cp->tempFactor[j]);/* 15 60-65 6    Real(6.2)    tempFactor Temperature factor. */
+        sscanf(&buff[72], "%4c",   cp->segID[j]);     /* 16 72-75 4    LString(4)   segID      Segment identifier, left-just. */
+        sscanf(&buff[76], "%2c",   cp->element[j]);   /* 17 76-77 2    LString(2)   element    Element symbol, right-just. */
+        sscanf(&buff[78], "%2c",   cp->charge[j]);    /* 18 78-79 2    LString(2)   charge     Charge on the atom. */
+    }
+    else /* standard PDB format http://www.rcsb.org/pdb/file_formats/pdb/pdbguide2.2/guide2.2_frame.html */
+    {
+        /*          1         2         3         4         5         6         7         8
+        012345678901234567890123456789012345678901234567890123456789012345678901234567890
+        ATOM    145  N   VAL A  25      32.433  16.336  57.540  1.00 11.92      A1   N
+        ATOM    146  CA  VAL A  25      31.132  16.439  58.160  1.00 11.85      A1   C  
+        ATOM  15088 1HG1 ILE    28      -3.430   4.303  -6.057  1.00  0.00           H  
+        ATOM   7580 HD23 LEU  1724     111.285  90.889 -61.535                      
+        */
+                                                      /* #  COLs  LEN  DATA TYPE    FIELD      DEFINITION */
+        sscanf(&buff[0],  "%6c",   cp->record[j]);    /* 1  0-5   6    Record name  "ATOM  " */
+        sscanf(&buff[6],  "%5u",  &cp->serial[j]);    /* 2  6-10  5    Integer      serial     Atom serial number. */
+        sscanf(&buff[12], "%1c",  &cp->Hnum[j]);      /* 3  12    1      hydrogen number, usu (!official). */
+        sscanf(&buff[13], "%3c",   cp->name[j]);      /* 4  13-15 4(3) Atom         name       Atom name. */
+        sscanf(&buff[16], "%1c",  &cp->altLoc[j]);    /* 5  16    1    Character    altLoc     Alternate location indicator. */
+        sscanf(&buff[17], "%3c",   cp->resName[j]);   /* 6  17-19 3    Residue name resName    Residue name. */
+        sscanf(&buff[20], "%1c",  &cp->xchainID[j]);  /* 7  20    1    Character    xchainID   Chain identifier (!official). */
+        sscanf(&buff[21], "%1c",  &cp->chainID[j]);   /* 8  21    1    Character    chainID    Chain identifier. */
+        sscanf(&buff[22], "%4d",  &cp->resSeq[j]);    /* 9  22-25 4    Integer      resSeq     Residue sequence number. */
+        sscanf(&buff[26], "%1c",  &cp->iCode[j]);     /* 10 26    1    AChar        iCode      Code for insertion of residues. */
+        sscanf(&buff[30], "%8lf", &cp->x[j]);         /* 11 30-37 8    Real(8.3)    x          Orthogonal coordinates for X */
+        sscanf(&buff[38], "%8lf", &cp->y[j]);         /* 12 30-37 8    Real(8.3)    y          Orthogonal coordinates for Y */
+        sscanf(&buff[46], "%8lf", &cp->z[j]);         /* 13 30-37 8    Real(8.3)    z          Orthogonal coordinates for Z */
+        sscanf(&buff[54], "%6lf", &cp->occupancy[j]); /* 14 54-59 6    Real(6.2)    occupancy  Occupancy. */
+        sscanf(&buff[60], "%6lf", &cp->tempFactor[j]);/* 15 60-65 6    Real(6.2)    tempFactor Temperature factor. */
+        sscanf(&buff[72], "%4c",   cp->segID[j]);     /* 16 72-75 4    LString(4)   segID      Segment identifier, left-just. */
+        sscanf(&buff[76], "%2c",   cp->element[j]);   /* 17 76-77 2    LString(2)   element    Element symbol, right-just. */
+        sscanf(&buff[78], "%2c",   cp->charge[j]);    /* 18 78-79 2    LString(2)   charge     Charge on the atom. */
+        
+        //printf("%4d:%4d %d %d \'%s\'\n", i, j, cp->serial[j], cp->Hnum[j], cp->name[j]);
+    }
+}
+
+
 /* reads all coordinate fields for each model in a pdb file */
 int
-ReadPDBCoords(char *pdbfile_name, PDBCoordsArray *pdbA, int cds_i,
-int modelnum)
+ReadPDBCoords(char *pdbfile_name, PDBCoordsArray *pdbA, int cds_i, int modelnum)
 {
     char            buff[99];
     int             i, j, pos;
-    char           *endline;
     FILE           *pdbfile = NULL;
     char            pdbdir[FILENAME_MAX], dirpdbfile_name[FILENAME_MAX];
-    PDBCoords     **coords = pdbA->coords;
+    PDBCoords     **cds = pdbA->coords;
+    PDBCoords      *cp = NULL;
 
     pdbfile = fopen(pdbfile_name, "r");
     if (pdbfile == NULL)
@@ -217,6 +288,8 @@ int modelnum)
     for (i = 0; i < modelnum; ++i)
     {
         pos = cds_i + i;
+        cp = cds[pos];
+
         /* move to next model */
         if (modelnum > 1)
         {
@@ -224,127 +297,41 @@ int modelnum)
             {
                 if (strncmp(buff, "MODEL", 5) == 0)
                 {
-                    pdbA->coords[pos]->model = i;
+                    cp->model = i;
                     break;
                 }
             }
         }
 
-        mystrncpy(pdbA->coords[pos]->filename, 
-                  pdbfile_name, FILENAME_MAX - 1);
+        mystrncpy(cp->filename, pdbfile_name, FILENAME_MAX - 1);
 
         /* read in the model's coords */
         j = 0;
         while (fgets(buff, sizeof(buff), pdbfile))
         {
-            if (j >= pdbA->coords[pos]->vlen
-                || strncmp(buff, "ENDMDL", 6) == 0
-                || strncmp(buff, "END   ", 6) == 0)
+            if (j >= cp->vlen ||
+                strncmp(buff, "ENDMDL", 6) == 0 ||
+                strncmp(buff, "END   ", 6) == 0)
             {
                 break;
             }
 
-            /* if the line is *not* an ATOM or HETATM record, just print it out to the file */
-            if (strncmp(buff, "ATOM  ", 6) == 0)
+            /* if the line is an ATOM or HETATM record */
+            if (strncmp(buff, "ATOM  ", 6) == 0 || 
+                strncmp(buff, "HETATM", 6) == 0)
             {
-                endline = strpbrk(&buff[54], "\r\n\f");
-                if (endline != NULL) /* kill the newline, if there */
-                    *endline = '\0';
+                ScanPDBLine(buff, cp, j);
+                XPLORcorrections(cp, j);
 
-                if (!isdigit(buff[12]) && !isspace(buff[12])) /* for nonstandard PDB files, with atom name in wrong column (shifted one to the left, such as with AMBER8) */
-                {
-                          /*         1         2         3         4         5         6         7         8
-                            12345678901234567890123456789012345678901234567890123456789012345678901234567890
-                            ATOM    145  N   VAL A  25      32.433  16.336  57.540  1.00 11.92      A1   N
-                            ATOM    146  CA  VAL A  25      31.132  16.439  58.160  1.00 11.85      A1   C  
-                            ATOM  15088 1HG1 ILE    28      -3.430   4.303  -6.057  1.00  0.00           H  
-                            ATOM   7580 HD23 LEU  1724     111.285  90.889 -61.535                      
-                          */
-                                                                                        /* #  COLs  LEN  DATA TYPE    FIELD      DEFINITION */
-                           sscanf(&buff[0],  "%6c",   coords[pos]->record[j]);    /* 1  0-5   6    Record name  "ATOM  "              */
-                           sscanf(&buff[6],  "%5u",  &coords[pos]->serial[j]);    /* 2  6-10  5    Integer      serial     Atom serial number. */
-                           sscanf(&buff[12], "%3c",   coords[pos]->name[j]);      /* 4  12-14 4(3) Atom         name       Atom name. -- unofficial, nonstandard PDB (AMBER8 format, hosers) */
-                           sscanf(&buff[15], "%1c",  &coords[pos]->Hnum[j]);      /* 3  15    1      hydrogen number, (!official). */
-                           sscanf(&buff[16], "%1c",  &coords[pos]->altLoc[j]);    /* 5  16    1    Character    altLoc     Alternate location indicator. */
-                           sscanf(&buff[17], "%3c",   coords[pos]->resName[j]);   /* 6  17-19 3    Residue name resName    Residue name. */
-                           sscanf(&buff[20], "%1c",  &coords[pos]->xchainID[j]);  /* 7  20    1    Character    xchainID   Chain identifier (!official). */
-                           sscanf(&buff[21], "%1c",  &coords[pos]->chainID[j]);   /* 8  21    1    Character    chainID    Chain identifier. */
-                           sscanf(&buff[22], "%4d",  &coords[pos]->resSeq[j]);    /* 9  22-25 4    Integer      resSeq     Residue sequence number. */
-                           sscanf(&buff[26], "%1c",  &coords[pos]->iCode[j]);     /* 10 26    1    AChar        iCode      Code for insertion of residues. */
-                           sscanf(&buff[30], "%8lf", &coords[pos]->x[j]);         /* 11 30-37 8    Real(8.3)    x          Orthogonal coordinates for X */
-                           sscanf(&buff[38], "%8lf", &coords[pos]->y[j]);         /* 12 30-37 8    Real(8.3)    y          Orthogonal coordinates for Y */
-                           sscanf(&buff[46], "%8lf", &coords[pos]->z[j]);         /* 13 30-37 8    Real(8.3)    z          Orthogonal coordinates for Z */
-                           sscanf(&buff[54], "%6lf", &coords[pos]->occupancy[j]); /* 14 54-59 6    Real(6.2)    occupancy  Occupancy. */
-                           sscanf(&buff[60], "%6lf", &coords[pos]->tempFactor[j]);/* 15 60-65 6    Real(6.2)    tempFactor Temperature factor. */
-                           sscanf(&buff[72], "%4c",   coords[pos]->segID[j]);     /* 16 72-75 4    LString(4)   segID      Segment identifier, left-just. */
-                           sscanf(&buff[76], "%2c",   coords[pos]->element[j]);   /* 17 76-77 2    LString(2)   element    Element symbol, right-just. */
-                           sscanf(&buff[78], "%2c",   coords[pos]->charge[j]);    /* 18 78-79 2    LString(2)   charge     Charge on the atom. */
-                }
-                else /* standard PDB format http://www.rcsb.org/pdb/file_formats/pdb/pdbguide2.2/guide2.2_frame.html */
-                {
-//                  sscanf(buff,
-//                         "%6c%5u%*1c%1c%3c%1c%3c%1c%1c%4d%1c%*1c%8lf%8lf%8lf%6lf%6lf%*6c%4c%2c%2c",
-//                                                           /* #  COLs  LEN  DATA TYPE    FIELD      DEFINITION */
-//                          coords[pos]->record[j],    /* 1  0-5   6    Record name  "ATOM  "              */
-//                         &coords[pos]->serial[j],    /* 2  6-10  5    Integer      serial     Atom serial number. */
-//                         &coords[pos]->Hnum[j],      /* 3  12    1      hydrogen number, usu (!official). */
-//                          coords[pos]->name[j],      /* 4  13-15 4(3) Atom         name       Atom name. */
-//                         &coords[pos]->altLoc[j],    /* 5  16    1    Character    altLoc     Alternate location indicator. */
-//                          coords[pos]->resName[j],   /* 6  17-19 3    Residue name resName    Residue name. */
-//                         &coords[pos]->xchainID[j],  /* 7  20    1    Character    xchainID   Chain identifier (!official). */
-//                         &coords[pos]->chainID[j],   /* 8  21    1    Character    chainID    Chain identifier. */
-//                         &coords[pos]->resSeq[j],    /* 9  22-25 4    Integer      resSeq     Residue sequence number. */
-//                         &coords[pos]->iCode[j],     /* 10 26    1    AChar        iCode      Code for insertion of residues. */
-//                         &coords[pos]->x[j],         /* 11 30-37 8    Real(8.3)    x          Orthogonal coordinates for X */
-//                         &coords[pos]->y[j],         /* 12 30-37 8    Real(8.3)    y          Orthogonal coordinates for Y */
-//                         &coords[pos]->z[j],         /* 13 30-37 8    Real(8.3)    z          Orthogonal coordinates for Z */
-//                         &coords[pos]->occupancy[j], /* 14 54-59 6    Real(6.2)    occupancy  Occupancy. */
-//                         &coords[pos]->tempFactor[j],/* 15 60-65 6    Real(6.2)    tempFactor Temperature factor. */
-//                          coords[pos]->segID[j],     /* 16 72-75 4    LString(4)   segID      Segment identifier, left-just. */
-//                          coords[pos]->element[j],   /* 17 76-77 2    LString(2)   element    Element symbol, right-just. */
-//                          coords[pos]->charge[j]);   /* 18 78-79 2    LString(2)   charge     Charge on the atom. */
-
-                          /*          1         2         3         4         5         6         7         8
-                            012345678901234567890123456789012345678901234567890123456789012345678901234567890
-                            ATOM    145  N   VAL A  25      32.433  16.336  57.540  1.00 11.92      A1   N
-                            ATOM    146  CA  VAL A  25      31.132  16.439  58.160  1.00 11.85      A1   C  
-                            ATOM  15088 1HG1 ILE    28      -3.430   4.303  -6.057  1.00  0.00           H  
-                            ATOM   7580 HD23 LEU  1724     111.285  90.889 -61.535                      
-                          */
-                                                                                        /* #  COLs  LEN  DATA TYPE    FIELD      DEFINITION */
-                           sscanf(&buff[0],  "%6c",   coords[pos]->record[j]);    /* 1  0-5   6    Record name  "ATOM  "              */
-                           sscanf(&buff[6],  "%5u",  &coords[pos]->serial[j]);    /* 2  6-10  5    Integer      serial     Atom serial number. */
-                           sscanf(&buff[12], "%1c",  &coords[pos]->Hnum[j]);      /* 3  12    1      hydrogen number, usu (!official). */
-                           sscanf(&buff[13], "%3c",   coords[pos]->name[j]);      /* 4  13-15 4(3) Atom         name       Atom name. */
-                           sscanf(&buff[16], "%1c",  &coords[pos]->altLoc[j]);    /* 5  16    1    Character    altLoc     Alternate location indicator. */
-                           sscanf(&buff[17], "%3c",   coords[pos]->resName[j]);   /* 6  17-19 3    Residue name resName    Residue name. */
-                           sscanf(&buff[20], "%1c",  &coords[pos]->xchainID[j]);  /* 7  20    1    Character    xchainID   Chain identifier (!official). */
-                           sscanf(&buff[21], "%1c",  &coords[pos]->chainID[j]);   /* 8  21    1    Character    chainID    Chain identifier. */
-                           sscanf(&buff[22], "%4d",  &coords[pos]->resSeq[j]);    /* 9  22-25 4    Integer      resSeq     Residue sequence number. */
-                           sscanf(&buff[26], "%1c",  &coords[pos]->iCode[j]);     /* 10 26    1    AChar        iCode      Code for insertion of residues. */
-                           sscanf(&buff[30], "%8lf", &coords[pos]->x[j]);         /* 11 30-37 8    Real(8.3)    x          Orthogonal coordinates for X */
-                           sscanf(&buff[38], "%8lf", &coords[pos]->y[j]);         /* 12 30-37 8    Real(8.3)    y          Orthogonal coordinates for Y */
-                           sscanf(&buff[46], "%8lf", &coords[pos]->z[j]);         /* 13 30-37 8    Real(8.3)    z          Orthogonal coordinates for Z */
-                           sscanf(&buff[54], "%6lf", &coords[pos]->occupancy[j]); /* 14 54-59 6    Real(6.2)    occupancy  Occupancy. */
-                           sscanf(&buff[60], "%6lf", &coords[pos]->tempFactor[j]);/* 15 60-65 6    Real(6.2)    tempFactor Temperature factor. */
-                           sscanf(&buff[72], "%4c",   coords[pos]->segID[j]);     /* 16 72-75 4    LString(4)   segID      Segment identifier, left-just. */
-                           sscanf(&buff[76], "%2c",   coords[pos]->element[j]);   /* 17 76-77 2    LString(2)   element    Element symbol, right-just. */
-                           sscanf(&buff[78], "%2c",   coords[pos]->charge[j]);    /* 18 78-79 2    LString(2)   charge     Charge on the atom. */
-
-                    //printf("%4d:%4d %d %d \'%s\'\n", i, j, coords[pos]->serial[j], coords[pos]->Hnum[j], coords[pos]->name[j]);
-                }
-
-                XPLORcorrections(coords[pos], j);
-
-                if (isfinite(coords[pos]->occupancy[j]) == 0)
-                    coords[pos]->occupancy[j] = 1.0;
+                if (isfinite(cp->occupancy[j]) == 0)
+                    cp->occupancy[j] = 1.0;
 
                 ++j;
             }
         }
     }
 
-    //PrintPDBCoords(stdout, pdbA->coords[0]);
+    //PrintPDBCoords(stdout, pdbA->cds[0]);
     fclose(pdbfile);
     return(EXIT_SUCCESS);
 }
@@ -448,14 +435,14 @@ int modelnum)
 /* } */
 
 
-CoordsArray
-*GetCoordsSelection(CoordsArray *baseA, PDBCoordsArray *pdbA)
+void
+GetCoordsSelection(CoordsArray *baseA, PDBCoordsArray *pdbA)
 {
     int             vlen;
     int            *selection_index = NULL; /* array of ints corresponding to selected atoms */
     int             i, j;
-    char           *chains; /* array of chars holding chain IDs of ranges in the selection criteria */
-    int            *upper, *lower; /* an array of ints holding the upper and lower range bounds */
+    char           *chains = NULL; /* array of chars holding chain IDs of ranges in the selection criteria */
+    int            *upper = NULL, *lower = NULL; /* an array of ints holding the upper and lower range bounds */
     int             selection_len;
     char          **endptr = NULL;
     int             range_num = 1; /* number of residue ranges specified in selection */
@@ -479,7 +466,7 @@ CoordsArray
         if (selections == NULL || lower == NULL || upper == NULL || chains == NULL)
         {
             perror("\n  ERROR");
-            fprintf(stderr, "\n  ERROR: could not allocate memory for selections in GetCoordsSelection(). \n");
+            fprintf(stderr, "\n  ERROR1000: could not allocate memory for selections in GetCoordsSelection(). \n");
             PrintTheseusTag();
             exit(EXIT_FAILURE);
         }
@@ -490,7 +477,7 @@ CoordsArray
             if (selections[i] == NULL)
             {
                 perror("\n  ERROR");
-                fprintf(stderr, "\n  ERROR: could not allocate memory for selections[] in GetCoordsSelection(). \n");
+                fprintf(stderr, "\n  ERROR1001: could not allocate memory for selections[] in GetCoordsSelection(). \n");
                 PrintTheseusTag();
                 exit(EXIT_FAILURE);
             }   
@@ -528,7 +515,7 @@ CoordsArray
 
             if (isalpha(selections[j][i]))
             {
-                fprintf(stderr, "\n  ERROR: incorrect format for chainID selection (too many characters). \n");
+                fprintf(stderr, "\n  ERROR1002: incorrect format for chainID selection (too many characters). \n");
                 Usage(0);
                 exit(EXIT_FAILURE);
             }
@@ -564,7 +551,7 @@ CoordsArray
         if (selections == NULL || lower == NULL || upper == NULL || chains == NULL || selections[0] == NULL)
         {
             perror("\n  ERROR");
-            fprintf(stderr, "\n  ERROR: could not allocate memory for selections in GetCoordsSelection(). \n");
+            fprintf(stderr, "\n  ERROR1003: could not allocate memory for selections in GetCoordsSelection(). \n");
             PrintTheseusTag();
             exit(EXIT_FAILURE);
         }
@@ -585,7 +572,7 @@ CoordsArray
     if (selection_index == NULL)
     {
         perror("\n  ERROR");
-        fprintf(stderr, "\n  ERROR: could not allocate memory for selection_index in GetCoordsSelection(). \n");
+        fprintf(stderr, "\n  ERROR1004: could not allocate memory for selection_index in GetCoordsSelection(). \n");
         PrintTheseusTag();
         exit(EXIT_FAILURE);
     }
@@ -602,10 +589,11 @@ CoordsArray
 /*                pdbA->coords[0]->altLoc[i], */
 /*                pdbA->coords[0]->chainID[i], */
 /*                pdbA->coords[0]->resSeq[i]); */
-            if (   strncmp(pdbA->coords[0]->record[i], "ATOM  ", 6) == 0
+            if (   (   strncmp(pdbA->coords[0]->record[i], "ATOM  ", 6) == 0
+                    || strncmp(pdbA->coords[0]->record[i], "HETATM", 6) == 0)
                 && atom_selxn(pdbA->coords[0]->name[i], baseA->algo->atoms, baseA->algo->atomslxn) == 1
                 && (pdbA->coords[0]->altLoc[i] == ' ' || pdbA->coords[0]->altLoc[i] == 'A')
-                && range_selxn(pdbA->coords[0]->chainID[i], chains, pdbA->coords[0]->resSeq[i], lower, upper, range_num) == 1) /* DLT FIX THIS!!!! */
+                && range_selxn(pdbA->coords[0]->chainID[i], chains, pdbA->coords[0]->resSeq[i], lower, upper, range_num) == 1) /* DLT debug FIX THIS!!!! */
             {
                 selection_index[vlen] = i;
                 ++vlen;
@@ -622,10 +610,11 @@ CoordsArray
 /*                pdbA->coords[0]->altLoc[i], */
 /*                pdbA->coords[0]->chainID[i], */
 /*                pdbA->coords[0]->resSeq[i]); */
-            if (   strncmp(pdbA->coords[0]->record[i], "ATOM  ", 6) == 0
+            if (   (   strncmp(pdbA->coords[0]->record[i], "ATOM  ", 6) == 0
+                    || strncmp(pdbA->coords[0]->record[i], "HETATM", 6) == 0)
                 && atom_selxn(pdbA->coords[0]->name[i], baseA->algo->atoms, baseA->algo->atomslxn) == 1
                 && (pdbA->coords[0]->altLoc[i] == ' ' || pdbA->coords[0]->altLoc[i] == 'A')
-                && range_selxn(pdbA->coords[0]->chainID[i], chains, pdbA->coords[0]->resSeq[i], lower, upper, range_num) == 0) /* DLT FIX THIS!!!! */
+                && range_selxn(pdbA->coords[0]->chainID[i], chains, pdbA->coords[0]->resSeq[i], lower, upper, range_num) == 0) /* DLT debug FIX THIS!!!! */
             {
                 selection_index[vlen] = i;
                 ++vlen;
@@ -677,8 +666,6 @@ CoordsArray
     free(upper);
     free(lower);
     free(chains);
-
-    return (baseA);
 }
 
 
@@ -745,7 +732,7 @@ PDBCoordsArray
                 {
                     if (strncmp(buff, "ENDMDL", 6) == 0)
                         break;
-                    else if (strncmp(buff, "ATOM  ", 6) == 0)
+                    else if (strncmp(buff, "ATOM  ", 6) == 0 || strncmp(buff, "HETATM", 6) == 0)
                         ++vlen;
                 }
 
@@ -755,15 +742,15 @@ PDBCoordsArray
                 else if ((models > 0) && (vlen != last_vlen))
                 {
                     fprintf(stderr,
-                            "\n  ERROR: coords #%d in \"%s\" has a different atom number than the preceding coords. \n",
-                            models + 1, pdbfile_name);
+                            "\n  ERROR1005: coords #%d in \"%s\" has a different atom number (%d) than the preceding coords (%d). \n",
+                            models + 1, pdbfile_name, vlen, last_vlen);
 /*                     PrintTheseusTag(); */
 /*                     exit(EXIT_FAILURE); */
                 }
                 else if (vlen < 3)
                 {
                     fprintf(stderr,
-                            "\n  ERROR: coords #%d in \"%s\" has no selected atoms. \n",
+                            "\n  ERROR1006: coords #%d in \"%s\" has no selected atoms. \n",
                             models + 1, pdbfile_name);
                     PrintTheseusTag();
                     exit(EXIT_FAILURE);
@@ -798,7 +785,7 @@ PDBCoordsArray
             {
                 if (strncmp(buff, "END   ", 6) == 0 || (strncmp(buff, "ENDMDL", 6) == 0))
                     break;
-                else if (strncmp(buff, "ATOM  ", 6) == 0)
+                else if (strncmp(buff, "ATOM  ", 6) == 0 || strncmp(buff, "HETATM", 6) == 0)
                     ++vlen;
             }
         }
@@ -850,9 +837,9 @@ PrintCoords(Coords *coords)
 {
     int             i;
 
-    fprintf(stderr, "\n veclen = %d \n", coords->vlen);
-    fprintf(stderr, " filename = %s \n", coords->filename);
-    fprintf(stderr, " model = %d \n",    coords->model);
+    fprintf(stdout, "\n veclen = %d \n", coords->vlen);
+    fprintf(stdout, " filename = %s \n", coords->filename);
+    fprintf(stdout, " model = %d \n",    coords->model);
 
     for (i = 0; i < coords->vlen; ++i)
     {
@@ -868,6 +855,7 @@ PrintCoords(Coords *coords)
     }
 
     printf(" END \n\n");
+    fflush(NULL);
 }
 
 
@@ -902,10 +890,13 @@ PrintPDBCoords(FILE *pdbfile, PDBCoords *pdbcoords)
         /* first expression evaluates whether at end of internal ATOM chain */
         /* second expression evaluates if at end of coords and ATOM chain */
         if (
-            ((i + 1) <  pdbcoords->vlen && strncmp(pdbcoords->record[i], "ATOM", 4) == 0 &&
+            ((i + 1) <  pdbcoords->vlen && 
+              ( strncmp(pdbcoords->record[i], "ATOM  ", 6) == 0 || strncmp(pdbcoords->record[i], "HETATM", 6) == 0 )
+            &&
              (pdbcoords->chainID[i] != pdbcoords->chainID[i+1] || pdbcoords->xchainID[i] != pdbcoords->xchainID[i+1]))
            ||
-            ((i + 1) == pdbcoords->vlen && strncmp(pdbcoords->record[i], "ATOM", 4) == 0)
+            ((i + 1) == pdbcoords->vlen && 
+             ( strncmp(pdbcoords->record[i], "ATOM  ", 6) == 0 || strncmp(pdbcoords->record[i], "HETATM", 6) == 0 ))
            ||
             (i < pdbcoords->vlen - 1 &&
             IsNameCAorP(pdbcoords->name[i]) &&
@@ -955,10 +946,14 @@ PrintOccPDBCoords(FILE *pdbfile, PDBCoords *pdbcoords)
             /* first expression evaluates whether at end of internal ATOM chain */
             /* second expression evaluates if at end of coords and ATOM chain */
             if (
-                ((i + 1) <  pdbcoords->vlen && strncmp(pdbcoords->record[i], "ATOM", 4) == 0 &&
+                ((i + 1) <  pdbcoords->vlen && 
+                 ( strncmp(pdbcoords->record[i], "ATOM  ", 6) == 0 || strncmp(pdbcoords->record[i], "HETATM", 6) == 0 )
+                &&
                  (pdbcoords->chainID[i] != pdbcoords->chainID[i+1] || pdbcoords->xchainID[i] != pdbcoords->xchainID[i+1]))
                ||
-                ((i + 1) == pdbcoords->vlen && strncmp(pdbcoords->record[i], "ATOM", 4) == 0)
+                ((i + 1) == pdbcoords->vlen 
+                && 
+                ( strncmp(pdbcoords->record[i], "ATOM  ", 6) == 0 || strncmp(pdbcoords->record[i], "HETATM", 6) == 0))
                ||
                 (i < pdbcoords->vlen - 1 &&
                 IsNameCAorP(pdbcoords->name[i]) &&
@@ -1154,6 +1149,40 @@ WriteTheseusModelFile(PDBCoordsArray *pdbA, Algorithm *algo, Statistics *stats,
 }
 
 
+void
+WriteTheseusPDBFiles(PDBCoordsArray *pdbA, Algorithm *algo, Statistics *stats)
+{
+    FILE           *pdbfile = NULL;
+    int             i;
+    char            outfile_name[FILENAME_MAX];
+
+    for (i = 0; i < pdbA->cnum; ++i)
+    {
+        strncpy(outfile_name, algo->rootname, strlen(algo->rootname)+1);
+        strcat(outfile_name, "_");
+        strncat(outfile_name, pdbA->coords[i]->filename, strlen(pdbA->coords[i]->filename)+1);
+        //strcpy(outfile_name, getroot(pdbA->coords[i]->filename));
+        //strcat(outfile_name, "_ths.pdb");
+
+        pdbfile = myfopen(outfile_name, "w");
+        if (pdbfile == NULL)
+        {
+            perror("\n  ERROR");
+            fprintf(stderr,
+                    "\n  ERROR99: could not open file '%s' for writing. \n", outfile_name);
+            PrintTheseusTag();
+            exit(EXIT_FAILURE);
+        }
+
+        PrintTheseusModelHeader(pdbfile);
+        PrintModelFileStats(pdbfile, pdbA, algo, stats);
+        PrintPDBCoords(pdbfile, pdbA->coords[i]);
+        fprintf(pdbfile, "END\n");
+        fclose(pdbfile);
+    }
+}
+
+
 void
 WriteOlveModelFile(PDBCoordsArray *pdbA, Algorithm *algo, Statistics *stats, char *outfile_name)
 {
@@ -1269,7 +1298,7 @@ WriteCoordsFile(Coords *coords, char *outfile_name)
     PDBCoordsArrayAlloc(pdbA, 0, coords->vlen);
     CopyCoords2PDB(pdbA->avecoords, coords);
     WritePDBCoordsFile(pdbA->avecoords, outfile_name);
-    PDBCoordsArrayDestroy(pdbA);
+    PDBCoordsArrayDestroy(&pdbA);
 }
 
 
@@ -1303,7 +1332,7 @@ WriteAveCoordsFile(CoordsArray *cdsA, char *outfile_name)
     PDBCoordsArrayAlloc(pdbA, 0, cdsA->vlen);
     CopyCoords2PDB(pdbA->avecoords, cdsA->avecoords);
     WriteAvePDBCoordsFile(pdbA, outfile_name);
-    PDBCoordsArrayDestroy(pdbA);
+    PDBCoordsArrayDestroy(&pdbA);
 }
 
 
