@@ -1,7 +1,7 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2007 Douglas L. Theobald
+    Copyright (C) 2004-2008 Douglas L. Theobald
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -90,8 +90,10 @@ Seq2pdbAlloc(Seq2PDB *seq2pdb, const int seqnum)
 
 
 void
-Seq2pdbDestroy(Seq2PDB *seq2pdb)
+Seq2pdbDestroy(Seq2PDB **seq2pdb_ptr)
 {
+    Seq2PDB *seq2pdb = *seq2pdb_ptr;
+
     free(seq2pdb->pdbfile_name_space);
     free(seq2pdb->seqname_space);
 
@@ -99,10 +101,10 @@ Seq2pdbDestroy(Seq2PDB *seq2pdb)
     free(seq2pdb->seqname);
     free(seq2pdb->map);
 
-    MSAfree(seq2pdb->msa);
+    MSAfree(&(seq2pdb->msa));
 
     free(seq2pdb);
-    seq2pdb = NULL;
+    *seq2pdb_ptr = NULL;
 }
 
 
@@ -152,7 +154,7 @@ Algorithm
     algo->revsel       = 0;
     algo->atoms        = 0;
     algo->reflection   = 0;
-    algo->embedave     = 2; /* 2 = biased ML embedded structure */
+    algo->embedave     = 0; /* 2 = biased ML embedded structure */
     algo->jackknife    = 0;
     algo->writestats   = 1;
     algo->random       = 0;
@@ -170,7 +172,7 @@ Algorithm
     algo->modelpca     = 0;
     algo->raxes[0] =
     algo->raxes[1] = 
-    algo->raxes[2] = 1.0;
+    algo->raxes[2] =   1.0;
     algo->mbias        = 0;
     algo->notrans      = 0;
     algo->norot        = 0;
@@ -179,7 +181,7 @@ Algorithm
     algo->dimweight    = 0;
     algo->covweight    = 0;
     algo->varweight    = 1;
-    algo->LedoitWolf   = 0/* -1 */;
+    algo->LedoitWolf   = 0;
     algo->hierarch     = 1;
     algo->leastsquares = 0;
     algo->filenum      = 0;
@@ -201,6 +203,13 @@ Algorithm
     algo->param[0] = algo->param[1] = 1.0;
     algo->radii[0] = algo->radii[1] = algo->radii[2] = 50.0;
     algo->ssm          = 0;
+    algo->lele5        = 0;
+    algo->bayes        = 0;
+    algo->ipmat        = 0;
+    algo->commandeur   = 0;
+    algo->missing      = 0;
+    algo->scale        = 0;
+    algo->instfile     = 0;
 
     return(algo);
 }
@@ -225,6 +234,8 @@ Statistics
     for (i = 0; i < 4; ++i)
         stats->skewness[i] = stats->kurtosis[i] = 0.0;
 
+    stats->hierarch_p1 = stats->hierarch_p2 = 1.0;
+
     return(stats);
 }
 
@@ -261,6 +272,7 @@ CoordsArray
     cdsA->w = NULL;
     cdsA->var = NULL;
     cdsA->df = NULL;
+    cdsA->S2 = NULL;
 
     cdsA->algo = AlgorithmInit();
     cdsA->stats = StatsInit();
@@ -365,96 +377,12 @@ CoordsArraySetup(CoordsArray *cdsA)
 
 
 void
-DistMatsAlloc(CoordsArray *cdsA)
-{
-    /* cdsA->Var_matrix = MatInit(cdsA->vlen, cdsA->vlen); */
-    cdsA->Dij_matrix = MatInit(cdsA->vlen, cdsA->vlen);
-    /* cdsA->distmat    = Mat3DInit(cdsA->cnum, cdsA->vlen, cdsA->vlen); */
-}
-
-
-void
-DistMatsDestroy(CoordsArray *cdsA)
-{
-/*     if (cdsA->Var_matrix != NULL) */
-/*         MatDestroy(cdsA->Var_matrix); */
-    if (cdsA->Dij_matrix != NULL)
-        MatDestroy(cdsA->Dij_matrix);
-/*     if (cdsA->distmat != NULL) */
-/*         Mat3DDestroy(cdsA->distmat); */
-}
-
-
-void
-CovMatsDestroy(CoordsArray *cdsA)
-{
-    if (cdsA->CovMat != NULL)
-        MatDestroy(cdsA->CovMat);
-    if (cdsA->FullCovMat != NULL)
-        MatDestroy(cdsA->FullCovMat);
-    if (cdsA->WtMat != NULL)
-        MatDestroy(cdsA->WtMat);
-    if (cdsA->tmpmatKK1 != NULL)
-        MatDestroy(cdsA->tmpmatKK1);
-    if (cdsA->tmpmatKK1 != NULL)
-        MatDestroy(cdsA->tmpmatKK2);
-    if (cdsA->tmpvecK != NULL)
-        free(cdsA->tmpvecK);
-    if (cdsA->tmpmat1 != NULL)
-        MatDestroy(cdsA->tmpmat1);
-    if (cdsA->tmpmat2 != NULL)
-        MatDestroy(cdsA->tmpmat2);
-    if (cdsA->tmpmat3a != NULL)
-        MatDestroy(cdsA->tmpmat3a);
-    if (cdsA->tmpmat3b != NULL)
-        MatDestroy(cdsA->tmpmat3b);
-    if (cdsA->tmpmat3c != NULL)
-        MatDestroy(cdsA->tmpmat3c);
-    if (cdsA->tmpmat3d != NULL)
-        MatDestroy(cdsA->tmpmat3d);
-    if (cdsA->MVCovMat != NULL)
-        MatDestroy(cdsA->MVCovMat);
-    if (cdsA->AxCovMat != NULL)
-        MatDestroy(cdsA->AxCovMat);
-    if (cdsA->AxWtMat != NULL)
-        MatDestroy(cdsA->AxWtMat);
-    if (cdsA->SCovMat != NULL)
-        MatDestroy(cdsA->SCovMat);
-    if (cdsA->tmpmatK3a != NULL)
-        MatDestroy(cdsA->tmpmatK3a);
-    if (cdsA->tmpmatK3b != NULL)
-        MatDestroy(cdsA->tmpmatK3b);
-    if (cdsA->tmpmat3K != NULL)
-        MatDestroy(cdsA->tmpmat3K);
-}
-
-
-void
-PCAAlloc(CoordsArray *cdsA)
-{
-    cdsA->pcamat = MatInit(cdsA->vlen, cdsA->vlen);
-    cdsA->pcavals = malloc(cdsA->vlen * sizeof(double));
-}
-
-
-void
-PCADestroy(CoordsArray *cdsA)
-{
-    if (cdsA->pcamat != NULL)
-        MatDestroy(cdsA->pcamat);
-    if (cdsA->pcavals != NULL)
-    {
-        free(cdsA->pcavals);
-        cdsA->pcavals = NULL;
-    }
-}
-
-
-void
-CoordsArrayDestroy(CoordsArray *cdsA)
+CoordsArrayDestroy(CoordsArray **cdsA_ptr)
 {
     int             i;
 
+    CoordsArray *cdsA = *cdsA_ptr;
+
     free(cdsA->msafile_name);
     cdsA->msafile_name = NULL;
     free(cdsA->mapfile_name);
@@ -462,15 +390,15 @@ CoordsArrayDestroy(CoordsArray *cdsA)
 
     for (i = 0; i < cdsA->cnum; ++i)
     {
-        CoordsDestroy(cdsA->coords[i]);
+        CoordsDestroy(&(cdsA->coords[i]));
     }
 
     free(cdsA->coords);
     cdsA->coords = NULL;
 
-    CoordsDestroy(cdsA->avecoords);
-    CoordsDestroy(cdsA->tcoords);
-    CoordsDestroy(cdsA->jkcoords);
+    CoordsDestroy(&(cdsA->avecoords));
+    CoordsDestroy(&(cdsA->tcoords));
+    CoordsDestroy(&(cdsA->jkcoords));
 
     free(cdsA->stats);
     cdsA->stats = NULL;
@@ -504,8 +432,101 @@ CoordsArrayDestroy(CoordsArray *cdsA)
     cdsA->var = NULL;
     free(cdsA->df);
     cdsA->df = NULL;
+
+    if (cdsA->S2 != NULL)
+    {
+        free(cdsA->S2);
+        cdsA->S2 = NULL;
+    }
+
     free(cdsA);
-    cdsA = NULL;
+    *cdsA_ptr = NULL;
+}
+
+
+void
+DistMatsAlloc(CoordsArray *cdsA)
+{
+    /* cdsA->Var_matrix = MatInit(cdsA->vlen, cdsA->vlen); */
+    cdsA->Dij_matrix = MatInit(cdsA->vlen, cdsA->vlen);
+    /* cdsA->distmat    = Mat3DInit(cdsA->cnum, cdsA->vlen, cdsA->vlen); */
+}
+
+
+void
+DistMatsDestroy(CoordsArray *cdsA)
+{
+/*     if (cdsA->Var_matrix != NULL) */
+/*         MatDestroy(&(cdsA->Var_matrix)); */
+    if (cdsA->Dij_matrix != NULL)
+        MatDestroy(&(cdsA->Dij_matrix));
+/*     if (cdsA->distmat != NULL) */
+/*         Mat3DDestroy(&(cdsA->distmat)); */
+}
+
+
+void
+CovMatsDestroy(CoordsArray *cdsA)
+{
+    if (cdsA->CovMat != NULL)
+        MatDestroy(&(cdsA->CovMat));
+    if (cdsA->FullCovMat != NULL)
+        MatDestroy(&(cdsA->FullCovMat));
+    if (cdsA->WtMat != NULL)
+        MatDestroy(&(cdsA->WtMat));
+    if (cdsA->tmpmatKK1 != NULL)
+        MatDestroy(&(cdsA->tmpmatKK1));
+    if (cdsA->tmpmatKK1 != NULL)
+        MatDestroy(&(cdsA->tmpmatKK2));
+    if (cdsA->tmpvecK != NULL)
+        free(cdsA->tmpvecK);
+    if (cdsA->tmpmat1 != NULL)
+        MatDestroy(&(cdsA->tmpmat1));
+    if (cdsA->tmpmat2 != NULL)
+        MatDestroy(&(cdsA->tmpmat2));
+    if (cdsA->tmpmat3a != NULL)
+        MatDestroy(&(cdsA->tmpmat3a));
+    if (cdsA->tmpmat3b != NULL)
+        MatDestroy(&(cdsA->tmpmat3b));
+    if (cdsA->tmpmat3c != NULL)
+        MatDestroy(&(cdsA->tmpmat3c));
+    if (cdsA->tmpmat3d != NULL)
+        MatDestroy(&(cdsA->tmpmat3d));
+    if (cdsA->MVCovMat != NULL)
+        MatDestroy(&(cdsA->MVCovMat));
+    if (cdsA->AxCovMat != NULL)
+        MatDestroy(&(cdsA->AxCovMat));
+    if (cdsA->AxWtMat != NULL)
+        MatDestroy(&(cdsA->AxWtMat));
+    if (cdsA->SCovMat != NULL)
+        MatDestroy(&(cdsA->SCovMat));
+    if (cdsA->tmpmatK3a != NULL)
+        MatDestroy(&(cdsA->tmpmatK3a));
+    if (cdsA->tmpmatK3b != NULL)
+        MatDestroy(&(cdsA->tmpmatK3b));
+    if (cdsA->tmpmat3K != NULL)
+        MatDestroy(&(cdsA->tmpmat3K));
+}
+
+
+void
+PCAAlloc(CoordsArray *cdsA)
+{
+    cdsA->pcamat = MatInit(cdsA->vlen, cdsA->vlen);
+    cdsA->pcavals = malloc(cdsA->vlen * sizeof(double));
+}
+
+
+void
+PCADestroy(CoordsArray *cdsA)
+{
+    if (cdsA->pcamat != NULL)
+        MatDestroy(&(cdsA->pcamat));
+    if (cdsA->pcavals != NULL)
+    {
+        free(cdsA->pcavals);
+        cdsA->pcavals = NULL;
+    }
 }
 
 
@@ -555,6 +576,7 @@ Coords
     coords->tmpmat3d = MatInit(3, 3);
 
     coords->bfact_c = 1.0;
+    coords->scale = 1.0;
 
     return(coords);
 }
@@ -632,8 +654,10 @@ CoordsSetup(Coords *coords)
 
 
 void
-CoordsDestroy(Coords *coords)
+CoordsDestroy(Coords **coords_ptr)
 {
+    Coords *coords = *coords_ptr;
+
     free(coords->resSeq);
     free(coords->resName_space);
     free(coords->chainID);
@@ -650,25 +674,25 @@ CoordsDestroy(Coords *coords)
     free(coords->covx);
     free(coords->covy);
     free(coords->covz);
-    MatDestroy(coords->matrix);
-    MatDestroy(coords->last_matrix);
-    MatDestroy(coords->jackmat);
+    MatDestroy(&(coords->matrix));
+    MatDestroy(&(coords->last_matrix));
+    MatDestroy(&(coords->jackmat));
 
     if (coords->innerprod != NULL)
-        MatDestroy(coords->innerprod);
+        MatDestroy(&(coords->innerprod));
 
     if (coords->innerprod2 != NULL)
-        MatDestroy(coords->innerprod2);
+        MatDestroy(&(coords->innerprod2));
 
-    MatDestroy(coords->evecs);
+    MatDestroy(&(coords->evecs));
 
-    MatDestroy(coords->tmpmat3a);
-    MatDestroy(coords->tmpmat3b);
-    MatDestroy(coords->tmpmat3c);
-    MatDestroy(coords->tmpmat3d);
+    MatDestroy(&(coords->tmpmat3a));
+    MatDestroy(&(coords->tmpmat3b));
+    MatDestroy(&(coords->tmpmat3c));
+    MatDestroy(&(coords->tmpmat3d));
 
     free(coords);
-    coords = NULL;
+    *coords_ptr = NULL;
 }
 
 
@@ -750,14 +774,15 @@ PDBCoordsArrayAllocLen(PDBCoordsArray *pdbA, const int vlen)
 
 
 void
-PDBCoordsArrayDestroy(PDBCoordsArray *pdbA)
+PDBCoordsArrayDestroy(PDBCoordsArray **pdbA_ptr)
 {
+    PDBCoordsArray *pdbA = *pdbA_ptr;
     int             i;
 
     for (i = 0; i < pdbA->cnum; ++i)
-        PDBCoordsDestroy(pdbA->coords[i]);
+        PDBCoordsDestroy(&(pdbA->coords[i]));
 
-    PDBCoordsDestroy(pdbA->avecoords);
+    PDBCoordsDestroy(&(pdbA->avecoords));
 
     if (pdbA->upper != NULL)
         free(pdbA->upper);
@@ -766,12 +791,12 @@ PDBCoordsArrayDestroy(PDBCoordsArray *pdbA)
         free(pdbA->lower);
 
     if (pdbA->seq2pdb != NULL)
-        Seq2pdbDestroy(pdbA->seq2pdb);
+        Seq2pdbDestroy(&(pdbA->seq2pdb));
 
     free(pdbA->coords);
     free(pdbA);
 
-    pdbA = NULL;
+    *pdbA_ptr = NULL;
 }
 
 
@@ -870,8 +895,10 @@ PDBCoordsAlloc(PDBCoords *pdbcoords, const int vlen)
 
 
 void
-PDBCoordsDestroy(PDBCoords *pdbcoords)
+PDBCoordsDestroy(PDBCoords **pdbcoords_ptr)
 {
+    PDBCoords *pdbcoords = *pdbcoords_ptr;
+
     free(pdbcoords->record_space);
     free(pdbcoords->name_space);
     free(pdbcoords->resName_space);
@@ -879,7 +906,7 @@ PDBCoordsDestroy(PDBCoords *pdbcoords)
     free(pdbcoords->element_space);
     free(pdbcoords->charge_space);
 
-    MatDestroy(pdbcoords->matrix);
+    MatDestroy(&(pdbcoords->matrix));
     free(pdbcoords->translation);
 
     free(pdbcoords->record);
@@ -902,5 +929,5 @@ PDBCoordsDestroy(PDBCoords *pdbcoords)
     free(pdbcoords->charge);
 
     free(pdbcoords);
-    pdbcoords = NULL;
+    *pdbcoords_ptr = NULL;
 }
