@@ -1,7 +1,7 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2007 Douglas L. Theobald
+    Copyright (C) 2004-2008 Douglas L. Theobald
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -41,7 +41,7 @@ void
 SSMAlloc(SSM *ssm, CoordsArray *cdsA);
 
 void
-SSMDestroy(SSM *ssm);
+SSMDestroy(SSM **ssm);
 
 void
 SSMCalc(SSM *ssm, CoordsArray *cdsA);
