@@ -1,7 +1,7 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2007 Douglas L. Theobald
+    Copyright (C) 2004-2008 Douglas L. Theobald
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -183,6 +183,8 @@ CalcStats(CoordsArray *incdsA)
         HierarchVars(cdsA);
         cdsA->algo->hierarch = tmph;
         //CalcLedoitCovMatPar(cdsA);
+        //#include "internmat.h"
+        //memcpy(&cdsA->CovMat[0][0], &internmat[0][0], vlen * vlen * sizeof(double));
         CalcPCA(cdsA); /* PCA analysis of covariance matrix */
     }
 
@@ -220,9 +222,9 @@ CalcStats(CoordsArray *incdsA)
     CopyStats(incdsA, cdsA);
 
     free(evals);
-    MatDestroy(rotmat);
-    MatDestroy(lastmat);
-    CoordsArrayDestroy(cdsA);
+    MatDestroy(&rotmat);
+    MatDestroy(&lastmat);
+    CoordsArrayDestroy(&cdsA);
 }
 
 
@@ -388,7 +390,7 @@ VarianceCoords(CoordsArray *cdsA)
 
     return(cdsA->stats->stddev);
 }
-            
+
 
 /* Calculates variances weighted by the inverse of a non-diagonal 3x3 dimensional covariance matrix */
 double
@@ -451,17 +453,18 @@ VarianceCoordsOcc(CoordsArray *cdsA)
     const int       cnum = cdsA->cnum, vlen = cdsA->vlen;
     double         *var = cdsA->var;
     const Coords  **coords = (const Coords **) cdsA->coords;
-    Coords         *coordsj;
+    Coords         *coordsj = NULL;
     const double   *axesw = cdsA->axesw;
     const double   *avex = (const double *) cdsA->avecoords->x,
                    *avey = (const double *) cdsA->avecoords->y,
                    *avez = (const double *) cdsA->avecoords->z;
-    double         occ;
+    double         occ, occsum;
 
     variance = 0.0;
     for (i = 0; i < vlen; ++i)
     {
         sqrx = sqry = sqrz = 0.0;
+        occsum = 0.0;
         for (j = 0; j < cnum; ++j)
         {
             coordsj = (Coords *) coords[j];
@@ -473,6 +476,8 @@ VarianceCoordsOcc(CoordsArray *cdsA)
             tmpz = coordsj->z[i] - avez[i];
             sqrz += occ * tmpz * tmpz;
 
+            occsum += occ;
+
 //         printf("\n%4d %4d %e - %e %e %e - %e %e %e",
 //                i, j, occ,
 //                coordsj->x[i], coordsj->y[i], coordsj->z[i],
@@ -481,18 +486,54 @@ VarianceCoordsOcc(CoordsArray *cdsA)
 
         sqrdist = (sqrx * axesw[0]) + (sqry * axesw[1]) + (sqrz * axesw[2]);
 
-        var[i] = sqrdist / (3.0 * cdsA->df[i]);
-        variance += sqrdist;
+        //var[i] = sqrdist / (3.0 * cdsA->df[i]);
+        var[i] = sqrdist / (3.0 * occsum); // should be the same as with df[i]
+        variance += var[i];
     }
 
     //WriteVariance(cdsA, "jacob.log"); exit(1);
-    variance /= (double) (vlen * cnum);
+    variance /= (double) vlen;
     cdsA->stats->stddev = sqrt(variance);
 
     return(cdsA->stats->stddev);
 }
 
 
+/* double */
+/* CalcOrderParams(CoordsArray *cdsA) */
+/* { */
+/*     int             i, j; */
+/*     double          tmpx, tmpy, tmpz; */
+/*     const int       cnum = cdsA->cnum, vlen = cdsA->vlen; */
+/*     const double    idf = 1.0 / (3 * cnum); */
+/*     double         *S2 = cdsA->S2; */
+/*     const Coords  **coords = (const Coords **) cdsA->coords; */
+/*     Coords         *coordsj; */
+/*     const double   *avex = (const double *) cdsA->avecoords->x, */
+/*                    *avey = (const double *) cdsA->avecoords->y, */
+/*                    *avez = (const double *) cdsA->avecoords->z; */
+/*  */
+/*     memset(S2, 0, vlen * sizeof(double)); */
+/*  */
+/*     for (i = 0; i < vlen; ++i) */
+/*     { */
+/*         coordsj = (Coords *) coords[j]; */
+/*  */
+/* 		for (j = 0; j < cnum; ++j) */
+/* 		{ */
+/* 			tmpx = coordsj->x[i] - avex[i]; */
+/* 			var[i] += tmpx * tmpx * axesw[0]; */
+/* 			tmpy = coordsj->y[i] - avey[i]; */
+/* 			var[i] += tmpy * tmpy * axesw[1]; */
+/* 			tmpz = coordsj->z[i] - avez[i]; */
+/* 			var[i] += tmpz * tmpz * axesw[2]; */
+/* 		} */
+/*     } */
+/*  */
+/*     return(cdsA->stats->stddev); */
+/* } */
+
+
 /* void */
 /* WriteTransformations(CoordsArray *cdsA, char *outfile_name) */
 /* { */
@@ -629,21 +670,20 @@ WriteVariance(CoordsArray *cdsA, char *outfile_name)
         exit(EXIT_FAILURE);
     }
 
-    fprintf(varfile, "#ATOM   resName resSeq     variance      std dev         RMSD pseudo-Bfact\n");
+    fprintf(varfile, "#ATOM   resName resSeq     variance      std dev         RMSD\n");
 
     if (cdsA->algo->varweight == 1 || cdsA->algo->leastsquares == 1)
     {
 		for (i = 0; i < cdsA->vlen; ++i)
 		{
 			fprintf(varfile,
-					"%-5d       %3s %6d %12.6f %12.6f %12.6f %12.6f\n",
+					"%-5d       %3s %6d %12.6f %12.6f %12.6f\n",
 					i+1,
 					cdsA->coords[0]->resName[i],
 					cdsA->coords[0]->resSeq[i],
 					cdsA->var[i],
 					sqrt(cdsA->var[i]),
-					sqrt(2.0 * cdsA->var[i] * (cnum - 1) / cnum),
-					cdsA->avecoords->b[i]);
+					sqrt(2.0 * cdsA->var[i] * (cnum - 1) / cnum));
 		}
     }
     else if (cdsA->algo->covweight == 1)
@@ -651,14 +691,13 @@ WriteVariance(CoordsArray *cdsA, char *outfile_name)
 		for (i = 0; i < cdsA->vlen; ++i)
 		{
 			fprintf(varfile,
-					"%-5d      %3s %6d %12.6f %12.6f %12.6f %12.6f\n",
+					"%-5d      %3s %6d %12.6f %12.6f %12.6f\n",
 					i+1,
 					cdsA->coords[0]->resName[i],
 					cdsA->coords[0]->resSeq[i],
 					cdsA->CovMat[i][i],
 					sqrt(cdsA->CovMat[i][i]),
-					sqrt(2.0 * cdsA->CovMat[i][i] * (cnum - 1) / cnum),
-					cdsA->avecoords->b[i]);
+					sqrt(2.0 * cdsA->CovMat[i][i] * (cnum - 1) / cnum));
 		}
     }
 
@@ -705,6 +744,52 @@ Bfacts2PrVars(CoordsArray *cdsA, int coord)
 }
 
 
+/* average of all possible unique pairwise RMSDs */
+/* Eqn 1 and following paragraph, Kearsley, S.K. (1990) "An algorithm for the 
+   simultaneous superposition of a structural series." Journal of Computational 
+   Chemistry 11(10):1187-1192. */
+/* double */
+/* CalcPRMSD(CoordsArray *cdsA) */
+/* { */
+/*     int             i, j, k; */
+/*     double          sqrdist; */
+/*     double          wsqrdist; */
+/*     double          paRMSD, pawRMSD; */
+/*     const int       cnum = cdsA->cnum, vlen = cdsA->vlen; */
+/*     const Coords  **coords = (const Coords **) cdsA->coords; */
+/*     const Coords   *coordsi, *coordsj; */
+/*     const double   *w = (const double *) cdsA->w; */
+/*     double   axesw[3]; */
+/*  */
+/*     for (i = 0; i < 3; ++i) */
+/*         axesw[i]= 1.0 / cdsA->AxCovMat[i][i]; */
+/*  */
+/*     sqrdist = wsqrdist = 0.0; */
+/*     for (i = 0; i < cnum; ++i) */
+/*     { */
+/*         coordsi = coords[i]; */
+/*  */
+/*         for (j = 0; j < i; ++j) */
+/*         { */
+/*             coordsj = coords[j]; */
+/*  */
+/*             for (k = 0; k < vlen; ++k) */
+/*             { */
+/*                 sqrdist += SqrCoordsDist(coordsi, k, coordsj, k); */
+/*                 wsqrdist += (w[k] * SqrCoordsDistMahal(coordsi, k, coordsj, k, axesw)); */
+/*             } */
+/*         } */
+/*     } */
+/*  */
+/*     paRMSD  = (2.0 *  sqrdist) / (double) (vlen * cnum * (cnum - 1)); */
+/*     pawRMSD = (2.0 * wsqrdist) / (double) (vlen * cnum * (cnum - 1)); */
+/*     cdsA->stats->ave_pawRMSD = sqrt(pawRMSD); */
+/*     cdsA->stats->ave_paRMSD  = sqrt(paRMSD); */
+/*  */
+/*     return (cdsA->stats->ave_paRMSD); */
+/* } */
+
+
 /* average of all possible unique pairwise RMSDs */
 /* Eqn 1 and following paragraph, Kearsley, S.K. (1990) "An algorithm for the 
    simultaneous superposition of a structural series." Journal of Computational 
@@ -715,6 +800,7 @@ CalcPRMSD(CoordsArray *cdsA)
     int             i, j, k;
     double          sqrdist;
     double          wsqrdist;
+    double          wtsum;
     double          paRMSD, pawRMSD;
     const int       cnum = cdsA->cnum, vlen = cdsA->vlen;
     const Coords  **coords = (const Coords **) cdsA->coords;
@@ -725,7 +811,7 @@ CalcPRMSD(CoordsArray *cdsA)
     for (i = 0; i < 3; ++i)
         axesw[i]= 1.0 / cdsA->AxCovMat[i][i];
 
-    sqrdist = wsqrdist = 0.0;
+    sqrdist = wsqrdist = wtsum = 0.0;
     for (i = 0; i < cnum; ++i)
     {
         coordsi = coords[i];
@@ -736,14 +822,22 @@ CalcPRMSD(CoordsArray *cdsA)
 
             for (k = 0; k < vlen; ++k)
             {
-                sqrdist += SqrCoordsDist(coordsi, k, coordsj, k);
-                wsqrdist += (w[k] * SqrCoordsDistMahal(coordsi, k, coordsj, k, axesw));
+                if (coordsi->o[k] > 0 && coordsj->o[k] > 0)
+                {
+                    wtsum += 1.0;
+                    sqrdist += SqrCoordsDist(coordsi, k, coordsj, k);
+                    wsqrdist += (w[k] * SqrCoordsDistMahal(coordsi, k, coordsj, k, axesw));
+                }
             }
         }
     }
 
-    paRMSD  = (2.0 *  sqrdist) / (double) (vlen * cnum * (cnum - 1));
-    pawRMSD = (2.0 * wsqrdist) / (double) (vlen * cnum * (cnum - 1));
+//    paRMSD  = (2.0 *  sqrdist) / (double) (vlen * cnum * (cnum - 1));
+//    printf("\npaRMSD = %8.3e\n", sqrt(paRMSD));
+    paRMSD  = sqrdist / wtsum; 
+//    printf("\npaRMSD = %8.3e\n", sqrt(paRMSD));
+    //pawRMSD = (2.0 * wsqrdist) / (double) (vlen * cnum * (cnum - 1));
+    pawRMSD = wsqrdist / wtsum;
     cdsA->stats->ave_pawRMSD = sqrt(pawRMSD);
     cdsA->stats->ave_paRMSD  = sqrt(paRMSD);
 
@@ -1183,7 +1277,7 @@ SkewnessCoords(CoordsArray *cdsA)
 
 /* find the multivariate normal skewness
    Mardia, K.V. (1970) "Measures of multivariate skewness and kurtosis with applications."
-   Biometrika 57, 519530.
+   Biometrika 57, 519-530.
 */
 
 
@@ -1321,7 +1415,7 @@ CalcANOVAF(CoordsArray *cdsA)
     cdsA->stats->anova_logL = anova->stats->logL;
     cdsA->stats->anova_AIC = anova->stats->AIC;
 
-    CoordsArrayDestroy(anova);
+    CoordsArrayDestroy(&anova);
     free(array1);
     free(array2);
 
@@ -1343,7 +1437,7 @@ CalcNormResidualsOld(CoordsArray *cdsA)
                    *avey = (const double *) cdsA->avecoords->y,
                    *avez = (const double *) cdsA->avecoords->z;
     double         *invvar = malloc(vlen * sizeof(double));
-    double         *normresid;
+    double         *normresid = NULL;
 
     if (cdsA->residuals == NULL)
         cdsA->residuals = malloc(vlen * 3 * cnum * sizeof(double));
@@ -1360,7 +1454,7 @@ CalcNormResidualsOld(CoordsArray *cdsA)
     /* memset(&SumMat[0][0], 0, 9 * sizeof(double)); */
     /* printf("\n%f %f %f", 1.0 / AxCovMat[0][0], 1.0 / AxCovMat[1][1], 1.0 / AxCovMat[2][2]); */
     j = 0;
-    
+
     for (k = 0; k < vlen; ++k)
     {
         rootv = sqrt(invvar[k]);
@@ -1395,7 +1489,7 @@ CalcNormResiduals(CoordsArray *cdsA)
     const double   *avex = (const double *) cdsA->avecoords->x,
                    *avey = (const double *) cdsA->avecoords->y,
                    *avez = (const double *) cdsA->avecoords->z;
-    double         *normresid;
+    double         *normresid = NULL;
 
     if (cdsA->residuals == NULL)
         cdsA->residuals = malloc(vlen * 3 * cnum * sizeof(double));
@@ -1450,7 +1544,7 @@ CalcNormResidualsLS(CoordsArray *cdsA)
     const double   *avex = (const double *) cdsA->avecoords->x,
                    *avey = (const double *) cdsA->avecoords->y,
                    *avez = (const double *) cdsA->avecoords->z;
-    double         *normresid;
+    double         *normresid = NULL;
 
     if (cdsA->residuals == NULL)
         cdsA->residuals = malloc(vlen * 3 * cnum * sizeof(double));
@@ -1497,7 +1591,7 @@ double
 FrobTerm(CoordsArray *cdsA)
 {
     int             i;
-    double          trace, logL;
+    double          trace;
     const int       cnum = cdsA->cnum, vlen = cdsA->vlen;
     const int       len = vlen * 3 * cnum;
     double         *residuals = cdsA->residuals;
@@ -1506,7 +1600,7 @@ FrobTerm(CoordsArray *cdsA)
     for (i = 0; i < len; ++i)
         trace += residuals[i] * residuals[i];
 
-    cdsA->stats->chi2 = chi_sqr_adapt(residuals, len, 0, &logL, 0.0, 1.0, normal_pdf, normal_lnpdf, normal_int);
+    //cdsA->stats->chi2 = chi_sqr_adapt(residuals, len, 0, &logL, 0.0, 1.0, normal_pdf, normal_lnpdf, normal_int);
 
     return(-0.5 * trace);
 }
@@ -1567,10 +1661,10 @@ FrobTerm2(CoordsArray *cdsA)
 
     trace = SumMat[0][0] + SumMat[1][1] + SumMat[2][2];
 
-    MatDestroy(ErrMat);
-    MatDestroy(SumMat);
-    MatDestroy(TmpMat);
-    MatDestroy(InvCovMat);
+    MatDestroy(&ErrMat);
+    MatDestroy(&SumMat);
+    MatDestroy(&TmpMat);
+    MatDestroy(&InvCovMat);
 
     return(-0.5 * trace);
 }
@@ -1583,21 +1677,69 @@ CalcHierarchLogL(CoordsArray *cdsA)
     Statistics     *stats = cdsA->stats;
     const int       vlen = cdsA->vlen, cnum = cdsA->cnum;
     const int       nd = cnum * 3;
-    /* double          logL; */
+    double          logL;
 
     switch(algo->hierarch)
     {
         case 0:
-        case 15:
-        case 16:
+        {
             return(0.0);
             break;
+        }
 
-        case 1: /* ML fit of variances to an inverse gamma distribution - 2 param */
+        case 1:
+        case 2:
+        case 7:
+        {
+            if (algo->varweight != 0)
+            {
+                double     *newvar = malloc(vlen * sizeof(double));
+                double      b, c, xn1;
+
+                b = stats->hierarch_p1;
+                c = stats->hierarch_p2;
+
+				memcpy(newvar, cdsA->var, vlen * sizeof(double));
+				qsort(newvar, vlen, sizeof(double), dblcmp_rev);
+				/* qsort-dblcmp_rev sorts big to small */
+				xn1 = newvar[vlen-2];
+
+                logL = invgamma_logL(newvar, vlen-1, b, c)
+                       //- b * ExpInvXn(xn1, b, c) - (1+c)*ExpLogXn(xn1, b, c)
+                       //-c * log(b) - lgamma(c)
+                       + log(invgamma_cdf(xn1, b, c));
+
+				free(newvar);
+
+				return(logL);
+				/* return(dist_logL(invgamma_lnpdf, stats->hierarch_p1, stats->hierarch_p2, cdsA->var, cdsA->vlen - 3)); */
+				/* invgamma_fit(newvar, cdsA->vlen - 3, &stats->hierarch_p1, &stats->hierarch_p12, &logL); */
+				/* return(cdsA->vlen * invgamma_logL(stats->hierarch_p1, stats->hierarch_p2)); */
+            }
+            else if (cdsA->algo->covweight != 0)
+            {
+				double **evecs = cdsA->tmpmatKK2;
+				int newlen;
+
+				if (vlen - 3 < nd - 6)
+					newlen = vlen - 3;
+				else
+					newlen = nd - 6;
+
+				eigenvalsym2((const double **) cdsA->CovMat, cdsA->var, evecs, vlen, &cdsA->tmpmatKK1[0][0]);
+				logL = invgamma_logL(cdsA->var + vlen - newlen, newlen, stats->hierarch_p1, stats->hierarch_p2);
+
+				return(logL);
+            }
+
+            break;
+        }
+
+        case 4: /* ML fit of variances to an inverse gamma distribution - 2 param */
+        {
             if (algo->varweight != 0)
             {
                 double *newvar = malloc(vlen * sizeof(double));
-                double logL;
 
 				memcpy(newvar, cdsA->var, vlen * sizeof(double));
 				qsort(newvar, vlen, sizeof(double), dblcmp_rev);
@@ -1613,7 +1755,6 @@ CalcHierarchLogL(CoordsArray *cdsA)
             {
 				double **evecs = cdsA->tmpmatKK2;
 				int newlen;
-				double logL;
 
 				if (vlen - 3 < nd - 6)
 					newlen = vlen - 3;
@@ -1625,24 +1766,29 @@ CalcHierarchLogL(CoordsArray *cdsA)
 
 				return(logL);
             }
-            break;
 
-        case 2:
+            break;
+        }
+
         case 3:
-        case 4:
         case 5:
         case 6:
-        case 7:
         case 8:
         case 9:
+        case 10:
         case 11:
         case 12:
         case 13:
         case 14:
+        case 15:
+        case 16:
         case 17:
         case 18:
+        case 19:
+        {
             return(invgamma_logL(cdsA->var, cdsA->vlen, stats->hierarch_p1, stats->hierarch_p2));
             break;
+        }
 
 //        case 8: /* Reciprocal Inverse Gaussian */ /* DLT debug */
 //            /* return(dist_logL(recinvgauss_lnpdf, stats->hierarch_p1, stats->hierarch_p2, cdsA->var, cdsA->vlen)); */
@@ -1657,10 +1803,12 @@ CalcHierarchLogL(CoordsArray *cdsA)
 //            break;
 
         default:
+        {
             printf("\n  ERROR:  Bad -g option \"%d\" \n", algo->hierarch);
             Usage(0);
             exit(EXIT_FAILURE);
             break;
+        }
     }
 
     return(0.0);
@@ -1690,7 +1838,7 @@ CalcHierarchLogL(CoordsArray *cdsA)
 double
 CalcLogL(CoordsArray *cdsA)
 {
-    const double    vlen = cdsA->vlen;
+    const int       vlen = cdsA->vlen;
     const double    cnum = cdsA->cnum;
     const double    nk = cnum * vlen;
     const double    nd = cnum * 3.0;
@@ -1718,18 +1866,36 @@ CalcLogL(CoordsArray *cdsA)
     }
     else if (algo->varweight == 1)
     {
-        lndetrow = 0.0;
-        for (i = 0; i < vlen; ++i)
-            lndetrow += log(var[i]);
-/* 		for (i = 0; i < vlen; ++i) */
-/* 			printf("\nevals: % 12.6e", var[i]); */
         if (algo->hierarch != 0)
         {
+            double *newvar = malloc(vlen * sizeof(double));
+            double xn1;
+
+            memcpy(newvar, var, vlen * sizeof(double));
+            qsort(newvar, vlen, sizeof(double), dblcmp_rev);
+            /* qsort-dblcmp_rev sorts big to small */
+
+            xn1 = newvar[vlen - 2];
+
+            lndetrow = 0.0;
+            for (i = 0; i < vlen-1; ++i)
+                lndetrow += log(newvar[i]);
+
+            lndetrow += ExpLogXn(stats->hierarch_p1, stats->hierarch_p2, xn1);
+
             frobterm = FrobTerm(cdsA);
             igL = CalcHierarchLogL(cdsA);
+
+            free(newvar);
         }
         else
+        {
+            lndetrow = 0.0;
+            for (i = 0; i < vlen; ++i)
+                lndetrow += log(var[i]);
+
             frobterm = -ndk2 /* FrobTerm(cdsA) */;
+        }
     }
     else if (algo->covweight == 1)
     {
@@ -1741,7 +1907,9 @@ CalcLogL(CoordsArray *cdsA)
             igL = CalcHierarchLogL(cdsA);
         }
         else
+        {
             frobterm = -ndk2;
+        }
     }
 
 /* printf("\n  * Axial variances:                [ %5.3f  %5.3f  %5.3f ] * ", */
@@ -1794,8 +1962,14 @@ CalcParamNum(CoordsArray *cdsA)
         case 0:
             break;
 
-        case 1: /* ML fit of variances to an inverse gamma distribution - 2 param */
+        case 1: /* ML fit of variances to an inverse gamma distribution - 1 param */
+            params += 1.0;
+            break;
+
+        /* ML fit of variances to an inverse gamma distribution - 2 param */
         case 2:
+        case 3:
+        case 4:
         case 5:
         case 6:
         case 7:
@@ -1807,11 +1981,6 @@ CalcParamNum(CoordsArray *cdsA)
         case 13:
             params += 2.0;
             break;
-
-        case 3: /* ML fit of variances to an inverse gamma distribution - 1 param */
-        case 4:
-            params += 1.0;
-            break;
     }
 
     /* for the mean */
@@ -1876,6 +2045,57 @@ TrCoordsInnerProd(Coords *coords, const int len)
 }
 
 
+double
+TrCoordsInnerProdWt(Coords *coords, const int len, const double *w)
+{
+    int             i;
+    double          sum, tmpx, tmpy, tmpz;
+
+    sum = 0.0;
+    for (i = 0; i < len; ++i)
+    {
+        tmpx = coords->x[i];
+        tmpy = coords->y[i];
+        tmpz = coords->z[i];
+        sum += w[i] * (tmpx*tmpx + tmpy*tmpy + tmpz*tmpz);
+    }
+
+    return(sum);
+}
+
+
+double
+TrCoordsInnerProd2(Coords *cds1, Coords *cds2, const int len)
+{
+    int             i;
+    double          sum;
+
+    sum = 0.0;
+    for (i = 0; i < len; ++i)
+    {
+        sum += (cds1->x[i]*cds2->x[i] + cds1->y[i]*cds2->y[i] + cds1->z[i]*cds2->z[i]);
+    }
+
+    return(sum);
+}
+
+
+double
+TrCoordsInnerProdWt2(Coords *cds1, Coords *cds2, const int len, const double *w)
+{
+    int             i;
+    double          sum;
+
+    sum = 0.0;
+    for (i = 0; i < len; ++i)
+    {
+        sum += w[i] * (cds1->x[i]*cds2->x[i] + cds1->y[i]*cds2->y[i] + cds1->z[i]*cds2->z[i]);
+    }
+
+    return(sum);
+}
+
+
 void
 UnbiasMean(CoordsArray *scratchA)
 {
