@@ -1,7 +1,7 @@
 /*
     Theseus - maximum likelihood superpositioning of macromolecular structures
 
-    Copyright (C) 2004-2007 Douglas L. Theobald
+    Copyright (C) 2004-2008 Douglas L. Theobald
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -312,9 +312,11 @@ CopyCoords2PDB(PDBCoords *pdbcoords, const Coords *coords)
             strncmp(coords->resName[i], "  G", 3) == 0 ||
             strncmp(coords->resName[i], "  T", 3) == 0 ||
             strncmp(coords->resName[i], " DA", 3) == 0 || /* remediated PDB residue names */
-            strncmp(coords->resName[i], " DC", 3) == 0 ||
-            strncmp(coords->resName[i], " DG", 3) == 0 ||
-            strncmp(coords->resName[i], " DT", 3) == 0 ||
+            strncmp(coords->resName[i], " DC", 3) == 0 || /* remediated PDB residue names */
+            strncmp(coords->resName[i], " DG", 3) == 0 || /* remediated PDB residue names */
+            strncmp(coords->resName[i], " DT", 3) == 0 || /* remediated PDB residue names */
+            strncmp(coords->resName[i], " DI", 3) == 0 || /* remediated PDB residue names */
+            strncmp(coords->resName[i], " DU", 3) == 0 || /* remediated PDB residue names */
             strncmp(coords->resName[i], "  U", 3) == 0)
             strcpy(pdbcoords->name[i], "P  ");
         else
@@ -606,23 +608,24 @@ CenMassWtOp(Coords *coords, const CoordsArray *weights)
     int             i;
     double          tempx, tempy, tempz;
     const double   *wts = (const double *) weights->w;
-    double          wti;
+    double          wti, wtsum;
     const double   *x = (const double *) coords->x,
                    *y = (const double *) coords->y,
                    *z = (const double *) coords->z;
 
-    tempx = tempy = tempz = 0.0;
+    tempx = tempy = tempz = wtsum = 0.0;
     for (i = 0; i < coords->vlen; ++i)
     {
         wti = wts[i];
+        wtsum += wti;
         tempx += (wti * x[i]);
         tempy += (wti * y[i]);
         tempz += (wti * z[i]);
     }
 
-    coords->center[0] = tempx / coords->vlen;
-    coords->center[1] = tempy / coords->vlen;
-    coords->center[2] = tempz / coords->vlen;
+    coords->center[0] = tempx / wtsum;
+    coords->center[1] = tempy / wtsum;
+    coords->center[2] = tempz / wtsum;
 
 /*     printf("\n********** %f %f %f", coords->center[0], coords->center[1], coords->center[2]); */
 /*     fflush(NULL); */
@@ -643,6 +646,8 @@ CenMassWtIp(Coords *coords, const double *wts)
     for (i = 0; i < coords->vlen; ++i)
     {
         wti = wts[i];
+//        printf("wt[%3d] = %8.3f\n", i, wti);
+
         wtsum += wti;
         tempx += (wti * x[i]);
         tempy += (wti * y[i]);
@@ -653,9 +658,13 @@ CenMassWtIp(Coords *coords, const double *wts)
     coords->center[1] = tempy / wtsum;
     coords->center[2] = tempz / wtsum;
 
-/*     printf("\n% 8.3f % 8.3f % 8.3f", */
-/*            coords->center[0], coords->center[1], coords->center[2]); */
+/*     printf("wtsum = %8.3f\n", wtsum); */
+/*     printf("wtocc = %8.3f\n", wtocc); */
 /*     fflush(NULL); */
+
+/*      printf("\nDT: % 8.3f % 8.3f % 8.3f\n",  */
+/*            coords->center[0], coords->center[1], coords->center[2]);  */
+/*      fflush(NULL);  */
 }
 
 
@@ -867,6 +876,79 @@ CenMassWtIpOcc(Coords *coords, const double *wts)
     coords->center[0] = tempx / wtsum;
     coords->center[1] = tempy / wtsum;
     coords->center[2] = tempz / wtsum;
+//    printf("wtsum = %8.3f\n", wtsum);
+//    fflush();
+
+/*     printf("\n% 8.3f % 8.3f % 8.3f",  */
+/*            coords->center[0], coords->center[1], coords->center[2]);  */
+/*     fflush(NULL); */ 
+}
+
+
+
+
+/*                 coordsi->x[j] = avex[j]*rmat00 + avey[j]*rmat01 + avez[j]*rmat02; */
+/*                 coordsi->y[j] = avex[j]*rmat10 + avey[j]*rmat11 + avez[j]*rmat12; */
+/*                 coordsi->z[j] = avex[j]*rmat20 + avey[j]*rmat21 + avez[j]*rmat22; */
+
+void
+CenMassWtIpEM(Coords *coords, const Coords *avecds, const double *wts)
+{
+    int             i;
+    double          tempx, tempy, tempz;
+    const double   *occ = (const double *) coords->o;
+    double          wti, wtsum;
+    const double   *x = (const double *) coords->x,
+                   *y = (const double *) coords->y,
+                   *z = (const double *) coords->z;
+    double          rmat00, rmat01, rmat02,
+                    rmat10, rmat11, rmat12,
+                    rmat20, rmat21, rmat22;
+    double        **rmat = coords->matrix;
+    double         *avex = avecds->x,
+                   *avey = avecds->y,
+                   *avez = avecds->z;
+
+    rmat00 = rmat[0][0], rmat01 = rmat[0][1], rmat02 = rmat[0][2],
+    rmat10 = rmat[1][0], rmat11 = rmat[1][1], rmat12 = rmat[1][2],
+    rmat20 = rmat[2][0], rmat21 = rmat[2][1], rmat22 = rmat[2][2];
+
+    tempx = tempy = tempz = wtsum = 0.0;
+    for (i = 0; i < coords->vlen; ++i)
+    {
+        wti = wts[i];
+//        printf("wt[%3d] = %8.3f\n", i, wti);
+        
+        if (occ[i] == 1.0)
+        {
+            //wtsum += 1.0;
+            wtsum += wti;
+            tempx += (wti * x[i]);
+            tempy += (wti * y[i]);
+            tempz += (wti * z[i]);
+        }
+        else if (occ[i] == 0.0)
+        {
+            tempx += (wti * (avex[i]*rmat00 + avey[i]*rmat01 + avez[i]*rmat02));
+            tempy += (wti * (avex[i]*rmat10 + avey[i]*rmat11 + avez[i]*rmat12));
+            tempz += (wti * (avex[i]*rmat20 + avey[i]*rmat21 + avez[i]*rmat22));
+
+/*             tempx += (wti * (avex[i]*rmat00 + avey[i]*rmat10 + avez[i]*rmat20)); */
+/*             tempy += (wti * (avex[i]*rmat01 + avey[i]*rmat11 + avez[i]*rmat21)); */
+/*             tempz += (wti * (avex[i]*rmat02 + avey[i]*rmat12 + avez[i]*rmat22)); */
+        }
+    }
+
+    coords->center[0] = tempx / wtsum;
+    coords->center[1] = tempy / wtsum;
+    coords->center[2] = tempz / wtsum;
+
+/*     printf("wtsum = %8.3f\n", wtsum); */
+/*     fflush(NULL); */
+
+/*     printf("\nEM: % 8.3f % 8.3f % 8.3f",  */
+/*            coords->center[0], coords->center[1], coords->center[2]);  */
+/*     fflush(NULL);  */
 }
 
 
@@ -1035,6 +1117,48 @@ EM_MissingCoords(CoordsArray *cdsA)
 }
 
 
+void
+EM_MissingCoordsOp(CoordsArray *baseA, CoordsArray *scratchA)
+{
+    int             i, j;
+    double        **rmat = NULL;
+    double         *avex = scratchA->avecoords->x,
+                   *avey = scratchA->avecoords->y,
+                   *avez = scratchA->avecoords->z;
+    const Coords  **coords = (const Coords **) baseA->coords;
+    Coords         *coordsi;
+
+    double          rmat00, rmat01, rmat02,
+                    rmat10, rmat11, rmat12,
+                    rmat20, rmat21, rmat22;
+
+    for (i = 0; i < baseA->cnum; ++i)
+    {
+        coordsi = (Coords *) coords[i];
+
+        rmat = scratchA->coords[i]->matrix;
+
+        rmat00 = rmat[0][0], rmat01 = rmat[0][1], rmat02 = rmat[0][2],
+        rmat10 = rmat[1][0], rmat11 = rmat[1][1], rmat12 = rmat[1][2],
+        rmat20 = rmat[2][0], rmat21 = rmat[2][1], rmat22 = rmat[2][2];
+
+        for (j = 0; j < baseA->vlen; ++j)
+        {
+            if (coordsi->o[j] == 0.0)
+            {
+                coordsi->x[j] = avex[j]*rmat00 + avey[j]*rmat01 + avez[j]*rmat02;
+                coordsi->y[j] = avex[j]*rmat10 + avey[j]*rmat11 + avez[j]*rmat12;
+                coordsi->z[j] = avex[j]*rmat20 + avey[j]*rmat21 + avez[j]*rmat22;
+
+                coordsi->x[j] -= scratchA->coords[i]->translation[0];
+                coordsi->y[j] -= scratchA->coords[i]->translation[1];
+                coordsi->z[j] -= scratchA->coords[i]->translation[2];
+            }
+        }
+    }
+}
+
+
 void
 AveCoordsNovec(CoordsArray *cdsA)
 {
@@ -1114,6 +1238,53 @@ AveCoords(CoordsArray *cdsA)
     }
 }
 
+void
+AveCoordsTB(CoordsArray *cdsA, int omit)
+{
+    int             i, j;
+    double         *avex = cdsA->avecoords->x,
+                   *avey = cdsA->avecoords->y,
+                   *avez = cdsA->avecoords->z,
+                   *aveo = cdsA->avecoords->o,
+                   *aveb = cdsA->avecoords->b;
+    const int       cnum = cdsA->cnum, vlen = cdsA->vlen;
+    const Coords  **coords = (const Coords **) cdsA->coords;
+    Coords         *coordsj;
+    double          invcnum = 1.0 / (double) (cnum-1);
+
+    memset(avex, 0, vlen * sizeof(double));
+    memset(avey, 0, vlen * sizeof(double));
+    memset(avez, 0, vlen * sizeof(double));
+    memset(aveb, 0, vlen * sizeof(double));
+
+    for (j = 0; j < cnum; ++j)
+    {
+    
+        if (j == omit)
+            continue;
+
+        coordsj = (Coords *) coords[j];
+
+		for (i = 0; i < vlen; ++i)
+		{
+			avex[i] += coordsj->x[i];
+			avey[i] += coordsj->y[i];
+			avez[i] += coordsj->z[i];
+			aveb[i] += coordsj->b[i];
+		}
+    }
+
+    for (i = 0; i < vlen; ++i)
+    {
+		avex[i] *= invcnum;
+		avey[i] *= invcnum;
+		avez[i] *= invcnum;
+		aveo[i] = 1.0;
+		aveb[i] *= invcnum;
+    }
+}
+
+
 
 static void
 *AveCds_pth(void *avedata_ptr)
@@ -1210,41 +1381,88 @@ AveCoords_pth(CoordsArray *cdsA, AveData **avedata, pthread_t *callThd,
 }
 
 
+/* void */
+/* AveCoordsOcc(CoordsArray *cdsA) */
+/* { */
+/*     int             i, j; */
+/*     double          xtmp, ytmp, ztmp, btmp, occ; */
+/*     double         *avex = cdsA->avecoords->x, */
+/*                    *avey = cdsA->avecoords->y, */
+/*                    *avez = cdsA->avecoords->z, */
+/*                    *aveo = cdsA->avecoords->o, */
+/*                    *aveb = cdsA->avecoords->b; */
+/*     const Coords  **coords = (const Coords **) cdsA->coords; */
+/*     Coords         *coordsj; */
+/*     const double    cnum = cdsA->cnum; */
+/*     double          invdf, occsum; */
+/*  */
+/*     for (i = 0; i < cdsA->vlen; ++i) */
+/*     { */
+/*         xtmp = ytmp = ztmp = btmp = 0.0; */
+/*         occsum = 0.0; */
+/*         for (j = 0; j < cnum; ++j) */
+/*         { */
+/*             coordsj = (Coords *) coords[j]; */
+/*             occ = coordsj->o[i]; */
+/*             occsum += occ; */
+/*             xtmp += occ * coordsj->x[i]; */
+/*             ytmp += occ * coordsj->y[i]; */
+/*             ztmp += occ * coordsj->z[i]; */
+/*             btmp += occ * coordsj->b[i]; */
+/*         } */
+/*  */
+/*         //invdf = 1.0 / cdsA->df[i]; */
+/*         invdf = 1.0 / occsum; */
+/*  */
+/*         avex[i] = xtmp * invdf; */
+/*         avey[i] = ytmp * invdf; */
+/*         avez[i] = ztmp * invdf; */
+/*         aveo[i] = 1.0; */
+/*         aveb[i] = btmp * invdf; */
+/*     } */
+/* } */
+
+
 void
 AveCoordsOcc(CoordsArray *cdsA)
 {
     int             i, j;
-    double          xtmp, ytmp, ztmp, btmp, occ;
     double         *avex = cdsA->avecoords->x,
                    *avey = cdsA->avecoords->y,
                    *avez = cdsA->avecoords->z,
                    *aveo = cdsA->avecoords->o,
                    *aveb = cdsA->avecoords->b;
+    const int       cnum = cdsA->cnum, vlen = cdsA->vlen;
     const Coords  **coords = (const Coords **) cdsA->coords;
     Coords         *coordsj;
-    const double    cnum = cdsA->cnum;
-    double          invdf;
+    double          occ, occsum, invocc;
 
-    for (i = 0; i < cdsA->vlen; ++i)
+    memset(avex, 0, vlen * sizeof(double));
+    memset(avey, 0, vlen * sizeof(double));
+    memset(avez, 0, vlen * sizeof(double));
+    memset(aveb, 0, vlen * sizeof(double));
+
+    for (i = 0; i < vlen; ++i)
     {
-        xtmp = ytmp = ztmp = btmp = 0.0;
+        occsum = 0.0;
         for (j = 0; j < cnum; ++j)
         {
             coordsj = (Coords *) coords[j];
             occ = coordsj->o[i];
-            xtmp += occ * coordsj->x[i];
-            ytmp += occ * coordsj->y[i];
-            ztmp += occ * coordsj->z[i];
-            btmp += occ * coordsj->b[i];
+            occsum += occ;
+			avex[i] += occ * coordsj->x[i];
+			avey[i] += occ * coordsj->y[i];
+			avez[i] += occ * coordsj->z[i];
+			aveb[i] += occ * coordsj->b[i];
         }
 
-        invdf = 1.0 / cdsA->df[i];
+        invocc = 1.0 / occsum;
 
-        avex[i] = xtmp * invdf;
-        avey[i] = ytmp * invdf;
-        avez[i] = ztmp * invdf;
+		avex[i] *= invocc;
+		avey[i] *= invocc;
+		avez[i] *= invocc;
         aveo[i] = 1.0;
-        aveb[i] = btmp * invdf;
+		aveb[i] *= invocc;
     }
 }
 
