@@ -16,7 +16,7 @@
  *                  dtheobald@gmail.com
  *                  dtheobald@brandeis.edu
  *
- *  Copyright:      Copyright (c) 2004-2007 Douglas L. Theobald
+ *  Copyright:      Copyright (c) 2004-2008 Douglas L. Theobald
  *
  *  THESEUS is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as 
@@ -44,10 +44,8 @@
  *          9/6/04 3:24 PM   Started source
  *  
  ******************************************************************/
-#include <pthread.h>
 #include "theseus.h"
-#include <dirent.h>
-#include <sys/stat.h>
+#include "theseuslib.h"
 
 
 #if defined(__APPLE__)
@@ -61,9 +59,11 @@
     static double           start_time, end_time;
 #endif
 
+/* global declarations necessary for leave(), I think */
 CoordsArray    *baseA = NULL; /* main array of selected pdb coords, never modified after read */
-PDBCoordsArray *pdbA = NULL; /* array holding all of the pdb file coordinate info, much of it unused in the actual calculations */
-   
+PDBCoordsArray *pdbA = NULL; /* array holding all of the pdb file coordinate info, 
+                                much of it unused in the actual calculations */
+
 static void
 leave(int sig);
 
@@ -74,7 +74,25 @@ int
 ConvertLele(char *fp_name, const int dim, const int forms, const int lmarks);
 
 void
-WriteLeleModelFile(PDBCoordsArray *pdbA);
+WriteLeleModelFile(PDBCoordsArray *pdbAr);
+
+
+void
+testcrush(CoordsArray *cdsA)
+{
+    int         i, j, m, n;
+
+    for (i = 0; i < cdsA->cnum; ++i)
+    {
+        for (j = 0; j < i; ++j)
+        {
+            for (m = 0; m < cdsA->coords[i]->vlen; ++m)
+                for (n = 0; n < cdsA->coords[j]->vlen; ++n)
+                    log(SqrCoordsDist(cdsA->coords[i], m, cdsA->coords[j], n));
+        }
+    }
+}
+
 
 int
 main(int argc, char *argv[])
@@ -97,10 +115,10 @@ main(int argc, char *argv[])
 /*     #define CHMOD744 (S_IRWXU | S_IROTH | S_IRGRP) */
 /*     mkdir("doug", CHMOD744); */
 
-#ifndef __MINGW32__
-    if (setrlimit(RLIMIT_CORE, 0) == 0)
-        printf("\n  WARNING2: could not set core limit size to 0. \n\n");
-#endif
+//#ifndef __MINGW32__
+//    if (setrlimit(RLIMIT_CORE, 0) == 0)
+//        printf("\n  WARNING2: could not set core limit size to 0. \n\n");
+//#endif
 
     baseA = CoordsArrayInit();
     algo = baseA->algo;
@@ -136,7 +154,7 @@ main(int argc, char *argv[])
         if (algo->random != '0')
         {
             strcat(algo->rootname, "_rand");
-            /* pdbA = GetPDBCoords(algo->infiles, algo->filenum, algo->fmodel); */
+            //pdbA = GetPDBCoords(algo->infiles, algo->filenum, algo->fmodel);
             if (algo->fmodel == 1)
                 pdbA = MakeRandPDBCoords(algo->iterations, /* # coordinates/forms */
                                          algo->jackknife, /* # of landmarks */
@@ -187,8 +205,8 @@ main(int argc, char *argv[])
 
         PrintTheseusTag();
 
-/*      CoordsArrayDestroy(baseA); */
-/*      PDBCoordsArrayDestroy(pdbA); */
+/*      CoordsArrayDestroy(&baseA); */
+/*      PDBCoordsArrayDestroy(&pdbA); */
 
         exit(EXIT_SUCCESS);
     }
@@ -250,7 +268,7 @@ main(int argc, char *argv[])
     {
         printf("    Selecting coordinates for superposition ... \n");
         fflush(NULL);
-    
+
         pdbA->vlen = NMRCheckPDBCoordsArray(pdbA);
         PDBCoordsArrayAllocLen(pdbA, pdbA->vlen);
         GetCoordsSelection(baseA, pdbA);
@@ -317,10 +335,13 @@ main(int argc, char *argv[])
             printf("    Using %d threads ... \n", algo->threads);
             fflush(NULL);
             MultiPose_pth(baseA);
+            //MultiPoseLib(baseA);
+            //test_charmm(baseA);
         }
         else
         {
             MultiPose(baseA);
+            //testcrush(baseA);
         }
     }
 
@@ -397,11 +418,14 @@ main(int argc, char *argv[])
 
         WriteTheseusModelFile(pdbA, algo, stats, mystrcat(algo->rootname, "_sup.pdb"));
 
+        if (algo->alignment == 1)
+            WriteTheseusPDBFiles(pdbA, algo, stats);
+
         if (algo->binary == 3 || algo->binary == 4)
         {
             printf("    Writing transformed coordinates binary file ... \n");
             fflush(NULL);
-    
+
             WriteBinPDBCoordsArray(pdbA);
         }
 
@@ -418,27 +442,31 @@ main(int argc, char *argv[])
         CenMass(baseA->avecoords);
         ApplyCenterIp(baseA->avecoords);
 
-        CoordsInnerProd(baseA->avecoords);
-        PrintCovMatGnuPlot((const double **) baseA->avecoords->innerprod, baseA->vlen, mystrcat(algo->rootname, "_mean_ip.mat"));
+        if (algo->ipmat == 1)
+        {
+            CoordsInnerProd(baseA->avecoords);
+            PrintCovMatGnuPlot((const double **) baseA->avecoords->innerprod, baseA->vlen, mystrcat(algo->rootname, "_mean_ip.mat"));
+        }
     }
 
     PrintTheseusTag();
 
-    CoordsArrayDestroy(baseA);
-    PDBCoordsArrayDestroy(pdbA);
+    CoordsArrayDestroy(&baseA);
+    PDBCoordsArrayDestroy(&pdbA);
 
     pthread_exit(NULL);
+
     return (EXIT_SUCCESS);
 }
 
 
 static void
-ParseCmdLine(int argc, char *argv[], CoordsArray *baseA)
+ParseCmdLine(int argc, char *argv[], CoordsArray *bseA)
 {
     int             option;
     extern char    *optarg;
     extern int      optind, opterr, optopt;
-    Algorithm      *algo = baseA->algo;
+    Algorithm      *algo = bseA->algo;
 
     int         i, cmdlinelen, argvlen;
 
@@ -461,7 +489,7 @@ ParseCmdLine(int argc, char *argv[], CoordsArray *baseA)
     }
 
     /*  */
-    while ((option = getopt(argc, argv, "a:A:b:B:cCdD:e:EfFg:GhHi:Ij:Jk:K:lL:m:M:nNo:Op:P:q:Qr:R:s:S:t:TuUvVw:WxXyYZz:0123:4:56")) != -1)
+    while ((option = getopt(argc, argv, "a:A:bB:cCd:D:e:EfFg:GhHi:IjJk:K:lL:m:M:nNo:Op:P:q:Qr:R:s:S:tT:uUvVw:WxXyYz:Z0123:4:5679")) != -1)
     {
         switch (option) /* See Algorithm structure in Coords.h for explanations of these flags/options */
         {
@@ -487,11 +515,22 @@ ParseCmdLine(int argc, char *argv[], CoordsArray *baseA)
                 algo->fmodel = 1;
                 break;
             case '5':
-                algo->fmodel = 1;
+                //algo->missing = 1;
+                algo->tenberge = 1;
                 break;
             case '6':
                 algo->ssm = 1;
                 break;
+            case '7': /* very specific fix for structured Lele 5x5 covariance matrix, testset data */
+                algo->lele5 = 1;
+                break;
+//            case '8':
+//                //algo->htrans = 1;
+//                algo->scale = 1;
+//                break;
+            case '9': /* write out the mean inner product matrix (Lele uses this) */
+                algo->ipmat = 1;
+                break;
             case 'a':
                 if (algo->alignment == 1)
                 {
@@ -500,6 +539,13 @@ ParseCmdLine(int argc, char *argv[], CoordsArray *baseA)
                     algo->atoms = 0;
                 }
 
+                if (algo->pca > 0)
+                {
+                    printf("\n\n    -> Only alpha carbons can be selected <-");
+                    printf("\n    -> when calculating principle components <-\n");
+                    algo->atoms = 0;
+                }
+
                 if (isdigit(optarg[0]))
                 {
                     algo->atoms = (int) strtol(optarg, NULL, 10);
@@ -519,17 +565,21 @@ ParseCmdLine(int argc, char *argv[], CoordsArray *baseA)
                 }
                 break;
             case 'A':
-                baseA->msafile_name = (char *) malloc((strlen(optarg) + 2) * sizeof(char));
-                mystrncpy(baseA->msafile_name, optarg, strlen(optarg) + 1);
+                bseA->msafile_name = (char *) malloc((strlen(optarg) + 2) * sizeof(char));
+                mystrncpy(bseA->msafile_name, optarg, strlen(optarg) + 1);
+
+                if (algo->atoms > 0)
+                {
+                    printf("\n\n    -> Only alpha carbons can be selected <-");
+                    printf("\n    -> when superimposing to an alignment <-\n");
+                    algo->atoms = 0;
+                }
+
                 algo->alignment = 1;
-                algo->atoms = 0;
                 algo->fmodel = 1;
                 break;
             case 'b':
-                if (isdigit(optarg[0]))
-                    algo->bfact = (int) strtol(optarg, NULL, 10);
-                if (algo->bfact > 0)
-                    algo->hierarch = 3;
+                algo->bayes = 1;
                 break;
             case 'B': /* 1 = read a binary PDB structure, 2 = write one */
                 algo->binary = (int) strtol(optarg, NULL, 10);
@@ -545,7 +595,8 @@ ParseCmdLine(int argc, char *argv[], CoordsArray *baseA)
                 algo->cormat = 0;
                 break;
             case 'd':
-                algo->dimweight = 1;
+                algo->scale = (double) strtod(optarg, NULL); /* specify a type of scaling to do */
+                //algo->dimweight = 1;
                 break;
             case 'D':
                 algo->FragDist = (int) strtol(optarg, NULL, 10);
@@ -584,10 +635,10 @@ ParseCmdLine(int argc, char *argv[], CoordsArray *baseA)
                 break;
             case 'I':
                 algo->info = 1;
-                //algo->write_file = 0;
                 break;
             case 'j':
-                algo->jackknife = (int) strtol(optarg, NULL, 10);
+                algo->instfile = 1;
+                //algo->jackknife = (int) strtol(optarg, NULL, 10);
                 break;
             case 'J': /* very expert options help message */
                 Usage(2);
@@ -644,8 +695,8 @@ ParseCmdLine(int argc, char *argv[], CoordsArray *baseA)
                 }
                 break;
             case 'M':
-                baseA->mapfile_name = (char *) malloc((strlen(optarg) + 2) * sizeof(char));
-                mystrncpy(baseA->mapfile_name, optarg, strlen(optarg) + 1);
+                bseA->mapfile_name = (char *) malloc((strlen(optarg) + 2) * sizeof(char));
+                mystrncpy(bseA->mapfile_name, optarg, strlen(optarg) + 1);
                 algo->alignment = 1;
                 algo->atoms = 0;
                 break;
@@ -656,8 +707,8 @@ ParseCmdLine(int argc, char *argv[], CoordsArray *baseA)
                 algo->nullrun = 1;
                 break;
             case 'o':
-                baseA->anchorf_name = (char *) malloc((strlen(optarg) + 2) * sizeof(char));
-                mystrncpy(baseA->anchorf_name, optarg, strlen(optarg) + 1);
+                bseA->anchorf_name = (char *) malloc((strlen(optarg) + 2) * sizeof(char));
+                mystrncpy(bseA->anchorf_name, optarg, strlen(optarg) + 1);
                 break;
             case 'O':
                 algo->olve = 1;
@@ -666,6 +717,13 @@ ParseCmdLine(int argc, char *argv[], CoordsArray *baseA)
                 algo->precision = (double) strtod(optarg, NULL);
                 break;
             case 'P':
+                if (algo->atoms > 0)
+                {
+                    printf("\n\n    -> Only alpha carbons can be selected <-");
+                    printf("\n    -> when calculating principle components <-\n");
+                    algo->atoms = 0;
+                }
+
                 algo->pca = (double) strtod(optarg, NULL);
                 break;
             case 'q':
@@ -693,10 +751,13 @@ ParseCmdLine(int argc, char *argv[], CoordsArray *baseA)
                 algo->revsel = 1;
                 break;
             case 't':
-                algo->threads = (int) strtol(optarg, NULL, 10);
+                if (isdigit(optarg[0]))
+                    algo->bfact = (int) strtol(optarg, NULL, 10);
+                if (algo->bfact > 0)
+                    algo->hierarch = 3;
                 break;
             case 'T':
-                algo->htrans = 1;
+                algo->threads = (int) strtol(optarg, NULL, 10);
                 break;
             case 'u':
                 algo->mbias = 1;
@@ -762,18 +823,6 @@ PrintSuperposStats(CoordsArray *cdsA)
     Statistics     *stats = cdsA->stats;
     Algorithm      *algo = cdsA->algo;
 
-    /* ave_wRMSD_from_mean = stats->ave_pawRMSD * sqrt((cnum - 1) / (2 * 3 * cnum)); */
-
-/*     double bias = 0.0; */
-/*     for (i = 0; i < vlen; ++i) */
-/*     { */
-/*         bias += */
-/*         cdsA->coords[0]->x[i] - cdsA->coords[1]->x[i] + */
-/*         cdsA->coords[0]->y[i] - cdsA->coords[1]->y[i] + */
-/*         cdsA->coords[0]->z[i] - cdsA->coords[1]->z[i]; */
-/*     } */
-/*     printf("\n  Bias %f:", bias); */
-
     if (algo->stats == 1)
     {
         if (algo->verbose == 1)
@@ -799,7 +848,6 @@ PrintSuperposStats(CoordsArray *cdsA)
 
     printf("    %d models superimposed in %.1f ms \n", cnum, algo->milliseconds);
 
-    /* printf("\n    Statistic                            before       after"); */
     printf("  * Least-squares <sigma>                        %10.5f\n", stats->stddev);
     printf("  * Classical LS pairwise <RMSD>                 %10.5f\n", stats->ave_paRMSD);
     /* printf("\n  * Weighted pairwise <RMSD>          %10.5f %10.5f * ", stats->starting_pawRMSD, stats->ave_pawRMSD); */
@@ -832,6 +880,7 @@ PrintSuperposStats(CoordsArray *cdsA)
         printf("  + Hierarchical var (%3.2e, %3.2e) chi^2 %11.2f (P:%3.2e)\n",
                stats->hierarch_p1, stats->hierarch_p2, stats->hierarch_chi2,
                chisqr_sdf(stats->hierarch_chi2 * newlen, newlen, 0));
+
         printf("  + Omnibus chi^2                               %11.2f (P:%3.2e)\n",
                (newlen * stats->hierarch_chi2 + vlen * cnum * 3 * stats->chi2) / (vlen * cnum * 3 + newlen),
                chisqr_sdf(newlen * stats->hierarch_chi2 + vlen * cnum * 3 * stats->chi2, (vlen * cnum * 3 + newlen), 0));
@@ -871,7 +920,7 @@ PrintSuperposStats(CoordsArray *cdsA)
         df2 = (cnum - 1) * Ftest_M;
 
         printf("  * Reflected <RMSD> from the mean             %10.6f\n", stats->anova_RMSD);
-        Pftest = InBeta((df2 / 2.0), (df1 / 2.0), (df2 / (df2 + df1 * F_ratio)));
+        Pftest = gsl_sf_beta_inc((df2 / 2.0), (df1 / 2.0), (df2 / (df2 + df1 * F_ratio)));
         printf("  * F(refl, %6d, %6d) = %8.4f       p = %3.2e\n", df1, df2, F_ratio, Pftest);
 
         logLF =   (df1 * log(1.0 + (df2 / (df1 * F_ratio))))
@@ -903,8 +952,8 @@ PrintSuperposStats(CoordsArray *cdsA)
     printf("  Total rounds = %d\n", algo->rounds + 1);
     if (algo->rounds + 1 >= algo->iterations)
     {
-        printf("\n\n   >> WARNING: Maximum iterations met. <<");
-        printf("\n   >> WARNING: Failure to converge to requested precision. <<\n");
+        printf("\n   >> WARNING: Maximum iterations met. <<");
+        printf("\n   >> WARNING: Failure to converge to requested precision. <<\n\n");
     }
     printf("  Converged to a fractional precision of %.1e\n", stats->precision);
     printf("I===-===-===-===-===-===-===-===-===-===-===-===-===-===-===-===-===-===-===-==I\n");
@@ -960,7 +1009,7 @@ ConvertLele_freeform(char *fp_name, const int dim, const int forms, const int lm
                 //printf("\n**** %f", vals[k]);
                 //fflush(NULL);
 
-                if (numscanned < 1)
+                if (numscanned < 1 || numscanned == EOF)
                 {
                     fprintf(stderr,
                             "\n ERROR6968: %d number of coordinates on line %d \n",
@@ -1036,18 +1085,18 @@ ConvertLele(char *fp_name, const int dim, const int forms, const int lmarks)
         {
             fgets(line, 512, fp0);
             numscanned = sscanf(line, "%le %le %le ", &vals[0], &vals[1], &vals[2]);
-            
-            if (numscanned < dim)
+
+            if (numscanned < dim || numscanned == EOF)
             {
                 fprintf(stderr,
                         "\n ERROR6968: %d number of coordinates on line %d \n",
                         numscanned, lines);
                 exit(EXIT_FAILURE);
             }
-    
+
                     /*  r     s     Hn  ar  xc r    i  x      y       z        o      tF     sI      e c */
                     /*  ATOM   1949 1HB  ARG A 255      19.326  -3.835  -3.438  1.00  1.31           H   */
-    
+
             fprintf(fp1,
                    /*     r  s   H    n  aL   rN  x  c  rSiC       x    y    z    o   tF          sI    e    c  */
                     "%-6.6s%5u  %3.3s %-3.3s %1c%4d    %8.3f%8.3f%8.3f%6.2f%6.2f\n",
@@ -1070,7 +1119,7 @@ ConvertLele(char *fp_name, const int dim, const int forms, const int lmarks)
 
 
 void
-WriteLeleModelFile(PDBCoordsArray *pdbA)
+WriteLeleModelFile(PDBCoordsArray *pdbAr)
 {
     FILE           *pdbfile = NULL;
     int             i, j;
@@ -1086,14 +1135,14 @@ WriteLeleModelFile(PDBCoordsArray *pdbA)
         exit(EXIT_FAILURE);
     }
 
-    for (i = 0; i < pdbA->cnum; ++i)
+    for (i = 0; i < pdbAr->cnum; ++i)
     {
-        for (j = 0; j < pdbA->vlen; ++j)
+        for (j = 0; j < pdbAr->vlen; ++j)
         {
             fprintf(pdbfile, "%.3f\t%.3f\t%.3f\n",
-                    pdbA->coords[i]->x[j],
-                    pdbA->coords[i]->y[j],
-                    pdbA->coords[i]->z[j]);
+                    pdbAr->coords[i]->x[j],
+                    pdbAr->coords[i]->y[j],
+                    pdbAr->coords[i]->z[j]);
         }
     }
 
